@@ -21,6 +21,7 @@ import java.awt.Component;
 import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.FileDialog;
+import java.awt.Frame;
 import java.awt.Toolkit;
 import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
@@ -32,7 +33,6 @@ import java.awt.event.KeyEvent;
 import java.awt.event.WindowEvent;
 import java.awt.print.PrinterException;
 import java.awt.print.PrinterJob;
-import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FilenameFilter;
 import java.io.IOException;
@@ -96,6 +96,7 @@ import org.apache.pdfbox.debugger.ui.DocumentEntry;
 import org.apache.pdfbox.debugger.ui.ErrorDialog;
 import org.apache.pdfbox.debugger.ui.ExtensionFileFilter;
 import org.apache.pdfbox.debugger.ui.FileOpenSaveDialog;
+import org.apache.pdfbox.debugger.ui.LogDialog;
 import org.apache.pdfbox.debugger.ui.MapEntry;
 import org.apache.pdfbox.debugger.ui.OSXAdapter;
 import org.apache.pdfbox.debugger.ui.PDFTreeCellRenderer;
@@ -105,6 +106,7 @@ import org.apache.pdfbox.debugger.ui.ReaderBottomPanel;
 import org.apache.pdfbox.debugger.ui.RecentFiles;
 import org.apache.pdfbox.debugger.ui.RotationMenu;
 import org.apache.pdfbox.debugger.ui.Tree;
+import org.apache.pdfbox.debugger.ui.WindowPrefs;
 import org.apache.pdfbox.debugger.ui.ZoomMenu;
 import org.apache.pdfbox.filter.FilterFactory;
 import org.apache.pdfbox.io.IOUtils;
@@ -125,7 +127,7 @@ import org.apache.pdfbox.printing.PDFPageable;
  * @author Ben Litchfield
  * @author Khyrul Bashar
  */
-@SuppressWarnings({"serial","squid:MaximumInheritanceDepth"})
+@SuppressWarnings({"serial","squid:MaximumInheritanceDepth","squid:S1948"})
 public class PDFDebugger extends JFrame
 {
     private static final Set<COSName> SPECIALCOLORSPACES =
@@ -135,6 +137,7 @@ public class PDFDebugger extends JFrame
             new HashSet<COSName>(Arrays.asList(COSName.ICCBASED, COSName.PATTERN, COSName.CALGRAY,
                                  COSName.CALRGB, COSName.LAB));
 
+    @SuppressWarnings({"squid:S2068"})
     private static final String PASSWORD = "-password";
     private static final String VIEW_STRUCTURE = "-viewstructure";
 
@@ -143,6 +146,7 @@ public class PDFDebugger extends JFrame
     
     private TreeStatusPane statusPane;
     private RecentFiles recentFiles;
+    private WindowPrefs windowPrefs;
     private boolean isPageMode;
 
     private PDDocument document;
@@ -151,10 +155,9 @@ public class PDFDebugger extends JFrame
     private static final String OS_NAME = System.getProperty("os.name").toLowerCase();
     private static final boolean IS_MAC_OS = OS_NAME.startsWith("mac os x");
     
-    private JScrollPane jScrollPane1;
-    private JScrollPane jScrollPane2;
-    private javax.swing.JSplitPane jSplitPane1;
-    private javax.swing.JTextPane jTextPane1;
+    private JScrollPane jScrollPaneRight;
+    private javax.swing.JSplitPane jSplitPane;
+    private javax.swing.JTextPane jTextPane;
     private ReaderBottomPanel statusBar;
     private Tree tree;
     private final JPanel documentPanel = new JPanel();
@@ -199,11 +202,11 @@ public class PDFDebugger extends JFrame
      */
     private void initComponents()
     {
-        jSplitPane1 = new javax.swing.JSplitPane();
-        jScrollPane1 = new JScrollPane();
+        jSplitPane = new javax.swing.JSplitPane();
+        JScrollPane jScrollPaneLeft = new JScrollPane();
         tree = new Tree(this);
-        jScrollPane2 = new JScrollPane();
-        jTextPane1 = new javax.swing.JTextPane();
+        jScrollPaneRight = new JScrollPane();
+        jTextPane = new javax.swing.JTextPane();
         
         tree.setCellRenderer(new PDFTreeCellRenderer());
         tree.setModel(null);
@@ -222,14 +225,14 @@ public class PDFDebugger extends JFrame
             @Override
             public void windowClosing(WindowEvent evt)
             {
-                exitForm(evt);
+                exitMenuItemActionPerformed(null);
             }
         });
-        
-        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
-        jScrollPane1.setBorder(new BevelBorder(BevelBorder.RAISED));
-        jScrollPane1.setPreferredSize(new Dimension(screenSize.width / 8, 500));
-        jSplitPane1.setDividerLocation(screenSize.width / 8);
+
+        windowPrefs = new WindowPrefs(this.getClass());
+
+        jScrollPaneLeft.setBorder(new BevelBorder(BevelBorder.RAISED));
+        jSplitPane.setDividerLocation(windowPrefs.getDividerLocation());
         tree.addTreeSelectionListener(new TreeSelectionListener()
         {
             @Override
@@ -239,15 +242,14 @@ public class PDFDebugger extends JFrame
             }
         });
 
-        jScrollPane1.setViewportView(tree);
+        jScrollPaneLeft.setViewportView(tree);
 
-        jSplitPane1.setRightComponent(jScrollPane2);
-        jSplitPane1.setDividerSize(3);
+        jSplitPane.setRightComponent(jScrollPaneRight);
+        jSplitPane.setDividerSize(3);
 
-        jScrollPane2.setPreferredSize(new Dimension(screenSize.width / 8 * 7, 500));
-        jScrollPane2.setViewportView(jTextPane1);
+        jScrollPaneRight.setViewportView(jTextPane);
 
-        jSplitPane1.setLeftComponent(jScrollPane1);
+        jSplitPane.setLeftComponent(jScrollPaneLeft);
 
         JScrollPane documentScroller = new JScrollPane();
         documentScroller.setViewportView(documentPanel);
@@ -257,7 +259,7 @@ public class PDFDebugger extends JFrame
         statusPane.getPanel().setPreferredSize(new Dimension(300, 25));
         getContentPane().add(statusPane.getPanel(), BorderLayout.PAGE_START);
 
-        getContentPane().add(jSplitPane1, BorderLayout.CENTER);
+        getContentPane().add(jSplitPane, BorderLayout.CENTER);
 
         statusBar = new ReaderBottomPanel();
         getContentPane().add(statusBar, BorderLayout.SOUTH);
@@ -269,7 +271,8 @@ public class PDFDebugger extends JFrame
         menuBar.add(createViewMenu());
         setJMenuBar(menuBar);
 
-        setBounds(screenSize.width / 4, screenSize.height / 4, screenSize.width / 2, screenSize.height / 2);
+        setExtendedState(windowPrefs.getExtendedState());
+        setBounds(windowPrefs.getBounds());
 
         // drag and drop to open files
         setTransferHandler(new TransferHandler()
@@ -707,12 +710,12 @@ public class PDFDebugger extends JFrame
                     showString(selectedNode);
                     return;
                 }
-                if (jSplitPane1.getRightComponent() == null
-                        || !jSplitPane1.getRightComponent().equals(jScrollPane2))
+                if (jSplitPane.getRightComponent() == null
+                        || !jSplitPane.getRightComponent().equals(jScrollPaneRight))
                 {
-                    replaceRightComponent(jScrollPane2);
+                    replaceRightComponent(jScrollPaneRight);
                 }
-                jTextPane1.setText(convertToString(selectedNode));
+                jTextPane.setText(convertToString(selectedNode));
             }
             catch (Exception e)
             {
@@ -974,7 +977,7 @@ public class PDFDebugger extends JFrame
         if (pane == null)
         {
             // unsupported font type
-            replaceRightComponent(jScrollPane2);
+            replaceRightComponent(jScrollPaneRight);
             return;
         }
         replaceRightComponent(pane);
@@ -983,9 +986,9 @@ public class PDFDebugger extends JFrame
     // replace the right component while keeping divider position
     private void replaceRightComponent(Component pane)
     {
-        int div = jSplitPane1.getDividerLocation();
-        jSplitPane1.setRightComponent(pane);
-        jSplitPane1.setDividerLocation(div);
+        int div = jSplitPane.getDividerLocation();
+        jSplitPane.setRightComponent(pane);
+        jSplitPane.setDividerLocation(div);
     }
 
     private void showString(Object selectedNode)
@@ -1064,18 +1067,21 @@ public class PDFDebugger extends JFrame
         }
         else if( selectedNode instanceof COSStream )
         {
+            COSStream stream = (COSStream) selectedNode;
+            InputStream in = null;
             try
             {
-                COSStream stream = (COSStream) selectedNode;
-                InputStream in = stream.createInputStream();
-                ByteArrayOutputStream baos = new ByteArrayOutputStream();
-                IOUtils.copy(in, baos);
-                data = baos.toString();
+                in = stream.createInputStream();
+                data = new String(IOUtils.toByteArray(in));
             }
-            catch( IOException e )
+            catch (IOException e)
             {
                 throw new RuntimeException(e);
             }
+            finally
+            {
+                IOUtils.closeQuietly(in);
+            }
         }
         else if( selectedNode instanceof MapEntry )
         {
@@ -1106,6 +1112,10 @@ public class PDFDebugger extends JFrame
                 throw new RuntimeException(e);
             }
         }
+        windowPrefs.setExtendedState(getExtendedState());
+        this.setExtendedState(Frame.NORMAL);
+        windowPrefs.setBounds(getBounds());
+        windowPrefs.setDividerLocation(jSplitPane.getDividerLocation());
         performApplicationExit();
     }
 
@@ -1172,14 +1182,6 @@ public class PDFDebugger extends JFrame
             throw new RuntimeException(e);
         }
     }
-    
-    /**
-     * Exit the Application.
-     */
-    private void exitForm(WindowEvent evt)
-    {
-        exitMenuItemActionPerformed(null);
-    }
 
     /**
      * Entry point.
@@ -1205,19 +1207,9 @@ public class PDFDebugger extends JFrame
             }
         });
 
-        // trigger premature initializations for more accurate rendering benchmarks
-        // See discussion in PDFBOX-3988
-        if (PDType1Font.COURIER.isStandard14())
-        {
-            // Yes this is always true
-            PDDeviceCMYK.INSTANCE.toRGB(new float[] { 0, 0, 0, 0} );
-            PDDeviceRGB.INSTANCE.toRGB(new float[] { 0, 0, 0 } );
-            IIORegistry.getDefaultInstance();
-            FilterFactory.INSTANCE.getFilter(COSName.FLATE_DECODE);
-        }
-
         // open file, if any
         String filename = null;
+        @SuppressWarnings({"squid:S2068"})
         String password = "";
         boolean viewPages = true;
         
@@ -1242,8 +1234,25 @@ public class PDFDebugger extends JFrame
             }
         }
         final PDFDebugger viewer = new PDFDebugger(viewPages);
-        
-        
+
+        // use our custom logger
+        // this works only if there is no "LogFactory.getLog()" in this class,
+        // and if there are no methods that call logging, even invisible
+        // use reduced file from PDFBOX-3653 to see logging
+        LogDialog.init(viewer, viewer.statusBar.getLogLabel());
+        System.setProperty("org.apache.commons.logging.Log", "org.apache.pdfbox.debugger.ui.DebugLog");
+
+        // trigger premature initializations for more accurate rendering benchmarks
+        // See discussion in PDFBOX-3988
+        if (PDType1Font.COURIER.isStandard14())
+        {
+            // Yes this is always true
+            PDDeviceCMYK.INSTANCE.toRGB(new float[] { 0, 0, 0, 0} );
+            PDDeviceRGB.INSTANCE.toRGB(new float[] { 0, 0, 0 } );
+            IIORegistry.getDefaultInstance();
+            FilterFactory.INSTANCE.getFilter(COSName.FLATE_DECODE);
+        }
+
         if (filename != null)
         {
             File file = new File(filename);
