@@ -62,7 +62,7 @@ class SimpleFont extends FontPane
         for (int index = 0; index <= 255; index++)
         {
             glyphs[index][0] = index;
-            if (font.getEncoding().contains(index))
+            if (font.getEncoding().contains(index) || font.toUnicode(index) != null)
             {
                 String glyphName = font.getEncoding().getName(index);
                 glyphs[index][1] = glyphName;
