@@ -29,7 +29,9 @@ import java.awt.geom.AffineTransform;
 import java.awt.image.BufferedImage;
 import java.io.IOException;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ExecutionException;
 import javax.swing.BoxLayout;
 import javax.swing.JLabel;
@@ -37,6 +39,8 @@ import javax.swing.JPanel;
 import javax.swing.SwingWorker;
 import javax.swing.event.AncestorEvent;
 import javax.swing.event.AncestorListener;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSDictionary;
 import org.apache.pdfbox.debugger.ui.ImageUtil;
 import org.apache.pdfbox.debugger.ui.RotationMenu;
@@ -47,7 +51,15 @@ import org.apache.pdfbox.rendering.PDFRenderer;
 import org.apache.pdfbox.debugger.PDFDebugger;
 import org.apache.pdfbox.debugger.ui.HighResolutionImageIcon;
 import org.apache.pdfbox.pdmodel.common.PDRectangle;
+import org.apache.pdfbox.pdmodel.interactive.action.PDAction;
+import org.apache.pdfbox.pdmodel.interactive.action.PDActionGoTo;
+import org.apache.pdfbox.pdmodel.interactive.action.PDActionURI;
+import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotation;
+import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotationLink;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotationWidget;
+import org.apache.pdfbox.pdmodel.interactive.documentnavigation.destination.PDDestination;
+import org.apache.pdfbox.pdmodel.interactive.documentnavigation.destination.PDNamedDestination;
+import org.apache.pdfbox.pdmodel.interactive.documentnavigation.destination.PDPageDestination;
 import org.apache.pdfbox.pdmodel.interactive.form.PDAcroForm;
 import org.apache.pdfbox.pdmodel.interactive.form.PDField;
 
@@ -59,6 +71,7 @@ import org.apache.pdfbox.pdmodel.interactive.form.PDField;
  */
 public class PagePane implements ActionListener, AncestorListener, MouseMotionListener, MouseListener
 {
+    private static final Log LOG = LogFactory.getLog(PagePane.class);
     private JPanel panel;
     private int pageIndex = -1;
     private final PDDocument document;
@@ -83,20 +96,85 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
     }
 
     private void initRectMap()
+    {
+        try
+        {
+            collectFieldLocations();
+            collectLinkLocations();
+        }
+        catch (IOException ex)
+        {
+            LOG.error(ex.getMessage(), ex);
+        }
+    }
+
+    private void collectLinkLocations() throws IOException
+    {
+        for (PDAnnotation annotation : page.getAnnotations())
+        {
+            if (annotation instanceof PDAnnotationLink)
+            {
+                collectLinkLocation((PDAnnotationLink) annotation);
+            }
+        }
+    }
+
+    private void collectLinkLocation(PDAnnotationLink linkAnnotation) throws IOException
+    {
+        PDAction action = linkAnnotation.getAction();
+        if (action instanceof PDActionURI)
+        {
+            PDActionURI uriAction = (PDActionURI) action;
+            rectMap.put(linkAnnotation.getRectangle(), "URI: " + uriAction.getURI());
+            return;
+        }
+        PDDestination destination;
+        if (action instanceof PDActionGoTo)
+        {
+            PDActionGoTo goToAction = (PDActionGoTo) action;
+            destination = goToAction.getDestination();
+        }
+        else
+        {
+            destination = linkAnnotation.getDestination();
+        }
+        if (destination instanceof PDNamedDestination)
+        {
+            destination = document.getDocumentCatalog().
+                    findNamedDestinationPage((PDNamedDestination) destination);
+        }
+        if (destination instanceof PDPageDestination)
+        {
+            PDPageDestination pageDestination = (PDPageDestination) destination;
+            int pageNum = pageDestination.retrievePageNumber();
+            if (pageNum != -1)
+            {
+                rectMap.put(linkAnnotation.getRectangle(), "Page destination: " + (pageNum + 1));
+            }
+        }
+    }
+
+    private void collectFieldLocations() throws IOException
     {
         PDAcroForm acroForm = document.getDocumentCatalog().getAcroForm();
         if (acroForm == null)
         {
             return;
         }
+        Set<COSDictionary> dictionarySet = new HashSet<COSDictionary>();
+        for (PDAnnotation annotation : page.getAnnotations())
+        {
+            dictionarySet.add(annotation.getCOSObject());
+        }
         for (PDField field : acroForm.getFieldTree())
         {
-            String fullyQualifiedName = field.getFullyQualifiedName();
             for (PDAnnotationWidget widget : field.getWidgets())
             {
-                if (page.equals(widget.getPage()))
+                // check if the annotation widget is on this page
+                // (checking widget.getPage() also works, but it is sometimes null)
+                if (dictionarySet.contains(widget.getCOSObject()))
                 {
-                    rectMap.put(widget.getRectangle(), fullyQualifiedName);
+                    rectMap.put(widget.getRectangle(), "Field name: " + field.getFullyQualifiedName());
                 }
             }
         }
@@ -196,11 +274,13 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
     @Override
     public void ancestorMoved(AncestorEvent ancestorEvent)
     {
+        // do nothing
     }
 
     @Override
     public void mouseDragged(MouseEvent e)
     {
+        // do nothing
     }
 
     /**
@@ -218,7 +298,8 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
         float zoomScale = zoomMenu.getPageZoomScale();
         float x = e.getX() / zoomScale * (float) defaultTransform.getScaleX();
         float y = e.getY() / zoomScale * (float) defaultTransform.getScaleY();
-        int x1, y1;
+        int x1;
+        int y1;
         switch ((RotationMenu.getRotationDegrees() + page.getRotation()) % 360)
         {
             case 90:
@@ -246,7 +327,7 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
         {
             if (entry.getKey().contains(x1, y1))
             {
-                text += ", field: " + rectMap.get(entry.getKey());
+                text += ", " + rectMap.get(entry.getKey());
                 break;
             }
         }
@@ -257,21 +338,25 @@ public class PagePane implements ActionListener, AncestorListener, MouseMotionLi
     @Override
     public void mouseClicked(MouseEvent e)
     {
+        // do nothing
     }
 
     @Override
     public void mousePressed(MouseEvent e)
     {
+        // do nothing
     }
 
     @Override
     public void mouseReleased(MouseEvent e)
     {
+        // do nothing
     }
 
     @Override
     public void mouseEntered(MouseEvent e)
     {
+        // do nothing
     }
 
     @Override
