@@ -20,6 +20,7 @@ import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.security.GeneralSecurityException;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
@@ -39,7 +40,6 @@ import java.util.Set;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSDictionary;
-import org.apache.pdfbox.cos.COSInputStream;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSObject;
 import org.apache.pdfbox.cos.COSStream;
@@ -47,16 +47,16 @@ import org.apache.pdfbox.cos.COSString;
 import org.apache.pdfbox.examples.signature.cert.CertificateVerificationException;
 import org.apache.pdfbox.examples.signature.cert.CertificateVerifier;
 import org.apache.pdfbox.io.IOUtils;
+import org.apache.pdfbox.io.RandomAccessBufferedFileInputStream;
+import org.apache.pdfbox.pdfparser.PDFParser;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDDocumentCatalog;
 import org.apache.pdfbox.pdmodel.encryption.SecurityProvider;
 import org.apache.pdfbox.pdmodel.interactive.digitalsignature.PDSignature;
+import org.apache.pdfbox.util.Charsets;
 import org.apache.pdfbox.util.Hex;
-import org.bouncycastle.asn1.ASN1Object;
 import org.bouncycastle.asn1.cms.Attribute;
-import org.bouncycastle.asn1.cms.AttributeTable;
 import org.bouncycastle.asn1.cms.CMSAttributes;
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.x509.Time;
 import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cert.jcajce.JcaCertStore;
@@ -66,11 +66,11 @@ import org.bouncycastle.cms.CMSProcessable;
 import org.bouncycastle.cms.CMSProcessableByteArray;
 import org.bouncycastle.cms.CMSSignedData;
 import org.bouncycastle.cms.SignerInformation;
-import org.bouncycastle.cms.SignerInformationVerifier;
 import org.bouncycastle.cms.jcajce.JcaSimpleSignerInfoVerifierBuilder;
 import org.bouncycastle.operator.OperatorCreationException;
 import org.bouncycastle.tsp.TSPException;
 import org.bouncycastle.tsp.TimeStampToken;
+import org.bouncycastle.util.CollectionStore;
 import org.bouncycastle.util.Selector;
 import org.bouncycastle.util.Store;
 
@@ -123,7 +123,16 @@ public final class ShowSignature
             PDDocument document = null;
             try
             {
-                document = PDDocument.load(infile, password);
+                // use old-style document loading to disable leniency
+                // see also https://www.pdf-insecurity.org/
+                RandomAccessBufferedFileInputStream raFile = new RandomAccessBufferedFileInputStream(infile);
+                // If your files are not too large, you can also download the PDF into a byte array
+                // with IOUtils.toByteArray() and pass a RandomAccessBuffer() object to the
+                // PDFParser constructor.
+                PDFParser parser = new PDFParser(raFile, password);
+                parser.setLenient(false);
+                parser.parse();
+                document = parser.getPDDocument();
                 for (PDSignature sig : document.getSignatureDictionaries())
                 {
                     COSDictionary sigDict = sig.getCOSObject();
@@ -134,7 +143,7 @@ public final class ShowSignature
                     byte[] buf = null;
                     try
                     {
-                        buf = sig.getSignedContent(fis);
+                        buf = sig.getSignedContent(fis); // alternatively, pass a byte array here
                     }
                     finally
                     {
@@ -143,30 +152,6 @@ public final class ShowSignature
 
                     System.out.println("Signature found");
 
-                    int[] byteRange = sig.getByteRange();
-                    if (byteRange.length != 4)
-                    {
-                        System.err.println("Signature byteRange must have 4 items");
-                    }
-                    else
-                    {
-                        long fileLen = infile.length();
-                        long rangeMax = byteRange[2] + (long) byteRange[3];
-                        // multiply content length with 2 (because it is in hex in the PDF) and add 2 for < and >
-                        int contentLen = contents.getString().length() * 2 + 2;
-                        if (fileLen != rangeMax || byteRange[0] != 0 || byteRange[1] + contentLen != byteRange[2])
-                        {
-                            // a false result doesn't necessarily mean that the PDF is a fake
-                            // see this answer why:
-                            // https://stackoverflow.com/a/48185913/535646
-                            System.out.println("Signature does not cover whole document");
-                        }
-                        else
-                        {
-                            System.out.println("Signature covers whole document");
-                        }
-                    }
-
                     if (sig.getName() != null)
                     {
                         System.out.println("Name:     " + sig.getName());
@@ -246,9 +231,9 @@ public final class ShowSignature
 
                                 if (sig.getSignDate() != null)
                                 {
-                                    verifyCertificateChain(new JcaCertStore(certs),
-                                            cert,
-                                            sig.getSignDate().getTime());
+                                    @SuppressWarnings("unchecked")
+                                    Store<X509CertificateHolder> store = new JcaCertStore(certs);
+                                    verifyCertificateChain(store, cert, sig.getSignDate().getTime());
                                 }
                             }
                         }
@@ -266,6 +251,31 @@ public final class ShowSignature
                     {
                         throw new IOException("Missing subfilter for cert dictionary");
                     }
+
+                    int[] byteRange = sig.getByteRange();
+                    if (byteRange.length != 4)
+                    {
+                        System.err.println("Signature byteRange must have 4 items");
+                    }
+                    else
+                    {
+                        long fileLen = infile.length();
+                        long rangeMax = byteRange[2] + (long) byteRange[3];
+                        // multiply content length with 2 (because it is in hex in the PDF) and add 2 for < and >
+                        int contentLen = contents.getString().length() * 2 + 2;
+                        if (fileLen != rangeMax || byteRange[0] != 0 || byteRange[1] + contentLen != byteRange[2])
+                        {
+                            // a false result doesn't necessarily mean that the PDF is a fake
+                            // see this answer why:
+                            // https://stackoverflow.com/a/48185913/535646
+                            System.out.println("Signature does not cover whole document");
+                        }
+                        else
+                        {
+                            System.out.println("Signature covers whole document");
+                        }
+                        checkContentValueWithFile(infile, byteRange, contents);
+                    }
                 }
                 analyseDSS(document);
             }
@@ -288,6 +298,58 @@ public final class ShowSignature
         }
     }
 
+    private void checkContentValueWithFile(File file, int[] byteRange, COSString contents) throws IOException
+    {
+        // https://stackoverflow.com/questions/55049270
+        // comment by mkl: check whether gap contains a hex value equal
+        // byte-by-byte to the Content value, to prevent attacker from using a literal string
+        // to allow extra space
+        RandomAccessBufferedFileInputStream raf = new RandomAccessBufferedFileInputStream(file);
+        raf.seek(byteRange[1]);
+        int c = raf.read();
+        if (c != '<')
+        {
+            System.err.println("'<' expected at offset " + byteRange[1] + ", but got " + (char) c);
+        }
+        byte[] contentFromFile = raf.readFully(byteRange[2] - byteRange[1] - 2);
+        byte[] contentAsHex = Hex.getString(contents.getBytes()).getBytes(Charsets.US_ASCII);
+        if (contentFromFile.length != contentAsHex.length)
+        {
+            System.err.println("Raw content length from file is " +
+                    contentFromFile.length +
+                    ", but internal content string in hex has length " +
+                    contentAsHex.length);
+        }
+        // Compare the two, we can't do byte comparison because of upper/lower case
+        // also check that it is really hex
+        for (int i = 0; i < contentFromFile.length; ++i)
+        {
+            try
+            {
+                if (Integer.parseInt(String.valueOf((char) contentFromFile[i]), 16) !=
+                    Integer.parseInt(String.valueOf((char) contentAsHex[i]), 16))
+                {
+                    System.err.println("Possible manipulation at file offset " +
+                                       (byteRange[1] + i + 1) + " in signature content");
+                    break;
+                }
+            }
+            catch (NumberFormatException ex)
+            {
+                System.err.println("Incorrect hex value");
+                System.err.println("Possible manipulation at file offset " +
+                                    (byteRange[1] + i + 1) + " in signature content");
+                break;
+            }
+        }
+        c = raf.read();
+        if (c != '>')
+        {
+            System.err.println("'>' expected at offset " + byteRange[2] + ", but got " + (char) c);
+        }
+        raf.close();
+    }
+
     private void verifyETSIdotRFC3161(byte[] buf, COSString contents)
             throws CertificateException, CMSException, IOException, OperatorCreationException,
             TSPException, NoSuchAlgorithmException, CertificateVerificationException
@@ -302,8 +364,7 @@ public final class ShowSignature
         System.out.println("certs=" + certs);
         
         String hashAlgorithm = timeStampToken.getTimeStampInfo().getMessageImprintAlgOID().getId();
-        // compare the hash of the signed content with the hash in
-        // the timestamp
+        // compare the hash of the signed content with the hash in the timestamp
         if (Arrays.equals(MessageDigest.getInstance(hashAlgorithm).digest(buf),
                 timeStampToken.getTimeStampInfo().getMessageImprintDigest()))
         {
@@ -316,7 +377,7 @@ public final class ShowSignature
 
         X509Certificate certFromTimeStamp = (X509Certificate) certs.iterator().next();
         SigUtils.checkTimeStampCertificateUsage(certFromTimeStamp);
-        validateTimestampToken(timeStampToken);
+        SigUtils.validateTimestampToken(timeStampToken);
                 verifyCertificateChain(timeStampToken.getCertificates(),
                 certFromTimeStamp,
                 timeStampToken.getTimeStampInfo().getGenTime());
@@ -345,11 +406,25 @@ public final class ShowSignature
         CMSSignedData signedData = new CMSSignedData(signedContent, contents.getBytes());
         @SuppressWarnings("unchecked")
         Store<X509CertificateHolder> certificatesStore = signedData.getCertificates();
+        if (certificatesStore.getMatches(null).isEmpty())
+        {
+            throw new IOException("No certificates in signature");
+        }
         Collection<SignerInformation> signers = signedData.getSignerInfos().getSigners();
+        if (signers.isEmpty())
+        {
+            throw new IOException("No signers in signature");
+        }
         SignerInformation signerInformation = signers.iterator().next();
         @SuppressWarnings("unchecked")
         Collection<X509CertificateHolder> matches =
                 certificatesStore.getMatches((Selector<X509CertificateHolder>) signerInformation.getSID());
+        if (matches.isEmpty())
+        {
+            throw new IOException("Signer '" + signerInformation.getSID().getIssuer() + 
+                                  ", serial# " + signerInformation.getSID().getSerialNumber() + 
+                                  " does not match any certificates");
+        }
         X509CertificateHolder certificateHolder = matches.iterator().next();
         X509Certificate certFromSignedData = new JcaX509CertificateConverter().getCertificate(certificateHolder);
         System.out.println("certFromSignedData: " + certFromSignedData);
@@ -357,19 +432,23 @@ public final class ShowSignature
         SigUtils.checkCertificateUsage(certFromSignedData);
 
         // Embedded timestamp
-        TimeStampToken timeStampToken = extractTimeStampTokenFromSignerInformation(signerInformation);
+        TimeStampToken timeStampToken = SigUtils.extractTimeStampTokenFromSignerInformation(signerInformation);
         if (timeStampToken != null)
         {
             // tested with QV_RCA1_RCA3_CPCPS_V4_11.pdf
             // https://www.quovadisglobal.com/~/media/Files/Repository/QV_RCA1_RCA3_CPCPS_V4_11.ashx
-            // timeStampToken.getCertificates() only contained the local certificate and not
-            // the whole chain, so use the store of the main signature.
-            // (If this assumption is incorrect, then the code must be changed to merge
-            // both stores, or to pass a collection)
-            validateTimestampToken(timeStampToken);
-            X509CertificateHolder tstCertHolder = (X509CertificateHolder) timeStampToken.getCertificates().getMatches(null).iterator().next();
+            // also 021496.pdf and 036351.pdf from digitalcorpora
+            SigUtils.validateTimestampToken(timeStampToken);
+            @SuppressWarnings("unchecked") // TimeStampToken.getSID() is untyped
+            Collection<X509CertificateHolder> tstMatches =
+                timeStampToken.getCertificates().getMatches((Selector<X509CertificateHolder>) timeStampToken.getSID());
+            X509CertificateHolder tstCertHolder = tstMatches.iterator().next();
             X509Certificate certFromTimeStamp = new JcaX509CertificateConverter().getCertificate(tstCertHolder);
-            verifyCertificateChain(certificatesStore,
+            // merge both stores using a set to remove duplicates
+            HashSet<X509CertificateHolder> certificateHolderSet = new HashSet<X509CertificateHolder>();
+            certificateHolderSet.addAll(certificatesStore.getMatches(null));
+            certificateHolderSet.addAll(timeStampToken.getCertificates().getMatches(null));
+            verifyCertificateChain(new CollectionStore<X509CertificateHolder>(certificateHolderSet),
                     certFromTimeStamp,
                     timeStampToken.getTimeStampInfo().getGenTime());
             SigUtils.checkTimeStampCertificateUsage(certFromTimeStamp);
@@ -449,22 +528,6 @@ public final class ShowSignature
         }
     }
 
-    private TimeStampToken extractTimeStampTokenFromSignerInformation(SignerInformation signerInformation)
-            throws CMSException, IOException, TSPException
-    {
-        if (signerInformation.getUnsignedAttributes() == null)
-        {
-            return null;
-        }
-        AttributeTable unsignedAttributes = signerInformation.getUnsignedAttributes();
-        // https://stackoverflow.com/questions/1647759/how-to-validate-if-a-signed-jar-contains-a-timestamp
-        Attribute attribute = unsignedAttributes.get(
-                PKCSObjectIdentifiers.id_aa_signatureTimeStampToken);
-        ASN1Object obj = (ASN1Object) attribute.getAttrValues().getObjectAt(0);
-        CMSSignedData signedTSTData = new CMSSignedData(obj.getEncoded());
-        return new TimeStampToken(signedTSTData);
-    }
-
     private void verifyCertificateChain(Store<X509CertificateHolder> certificatesStore,
             X509Certificate certFromSignedData, Date signDate)
             throws CertificateVerificationException, CertificateException
@@ -485,19 +548,10 @@ public final class ShowSignature
             }
         }
         CertificateVerifier.verifyCertificate(certFromSignedData, additionalCerts, true, signDate);
-    }
-
-    private void validateTimestampToken(TimeStampToken timeStampToken)
-            throws IOException, CertificateException, TSPException, OperatorCreationException
-    {
-        // https://stackoverflow.com/questions/42114742/
-        Collection<X509CertificateHolder> tstMatches =
-                timeStampToken.getCertificates().getMatches(timeStampToken.getSID());
-        X509CertificateHolder holder = tstMatches.iterator().next();
-        X509Certificate tstCert = new JcaX509CertificateConverter().getCertificate(holder);
-        SignerInformationVerifier siv = new JcaSimpleSignerInfoVerifierBuilder().setProvider(SecurityProvider.getProvider()).build(tstCert);
-        timeStampToken.validate(siv);
-        System.out.println("TimeStampToken validated");
+        //TODO check whether the root certificate is in our trusted list.
+        // For the EU, get a list here:
+        // https://ec.europa.eu/digital-single-market/en/eu-trusted-lists-trust-service-providers
+        // ( getRootCertificates() is not helpful because these are SSL certificates)
     }
 
     /**
@@ -550,8 +604,9 @@ public final class ShowSignature
             {
                 COSStream cosStream = (COSStream) streamObj.getObject();
 
-                COSInputStream input = cosStream.createInputStream();
+                InputStream input = cosStream.createInputStream();
                 byte[] streamBytes = IOUtils.toByteArray(input);
+                input.close();
 
                 System.out.println(description + " (" + elements.indexOf(streamObj) + "): "
                         + Hex.getString(streamBytes));
@@ -565,6 +620,8 @@ public final class ShowSignature
     private static void usage()
     {
         System.err.println( "usage: java " + ShowSignature.class.getName() +
-                            "<password> <inputfile>" );
+                            " <password (usually empty)> <inputfile>" );
+        // The password is for encrypted files and has nothing to do with the signature.
+        // (A PDF can be both encrypted and signed)
     }
 }
