@@ -49,11 +49,11 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.encryption.SecurityProvider;
-import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Primitive;
 import org.bouncycastle.asn1.ASN1Sequence;
-import org.bouncycastle.asn1.DEROctetString;
-import org.bouncycastle.asn1.DERTaggedObject;
+import org.bouncycastle.asn1.ASN1TaggedObject;
 import org.bouncycastle.asn1.ocsp.OCSPObjectIdentifiers;
 import org.bouncycastle.asn1.x509.Extension;
 import org.bouncycastle.asn1.x509.GeneralName;
@@ -297,13 +297,13 @@ public final class CertificateVerifier
         {
             // AccessDescription
             ASN1Sequence obj = (ASN1Sequence) objects.nextElement();
-            ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier) obj.getObjectAt(0);
-            if (!oid.equals(X509ObjectIdentifiers.id_ad_caIssuers))
+            ASN1Encodable oid = obj.getObjectAt(0);
+            if (!X509ObjectIdentifiers.id_ad_caIssuers.equals(oid))
             {
                 continue;
             }
-            DERTaggedObject location = (DERTaggedObject) obj.getObjectAt(1);
-            DEROctetString uri = (DEROctetString) location.getObject();
+            ASN1TaggedObject location = (ASN1TaggedObject) obj.getObjectAt(1);
+            ASN1OctetString uri = (ASN1OctetString) location.getObject();
             InputStream in = null;
             try
             {
@@ -411,13 +411,13 @@ public final class CertificateVerifier
             {
                 // AccessDescription
                 ASN1Sequence obj = (ASN1Sequence) objects.nextElement();
-                ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier) obj.getObjectAt(0);
+                ASN1Encodable oid = obj.getObjectAt(0);
                 // accessLocation
-                DERTaggedObject location = (DERTaggedObject) obj.getObjectAt(1);
-                if (oid.equals(X509ObjectIdentifiers.id_ad_ocsp)
+                ASN1TaggedObject location = (ASN1TaggedObject) obj.getObjectAt(1);
+                if (X509ObjectIdentifiers.id_ad_ocsp.equals(oid)
                         && location.getTagNo() == GeneralName.uniformResourceIdentifier)
                 {
-                    DEROctetString url = (DEROctetString) location.getObject();
+                    ASN1OctetString url = (ASN1OctetString) location.getObject();
                     String ocspURL = new String(url.getOctets());
                     LOG.info("OCSP URL: " + ocspURL);
                     return ocspURL;
@@ -463,6 +463,12 @@ public final class CertificateVerifier
             return;
         }
 
+        if (ocspHelper.getCertificateToCheck().equals(ocspResponderCertificate))
+        {
+            LOG.info("OCSP responder certificate is identical to certificate to check");
+            return;
+        }
+
         LOG.info("Check of OCSP responder certificate");
         Set<X509Certificate> additionalCerts2 = new HashSet<X509Certificate>(additionalCerts);
         JcaX509CertificateConverter certificateConverter = new JcaX509CertificateConverter();
