@@ -24,11 +24,13 @@ import java.net.HttpURLConnection;
 import java.net.URL;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
+import java.security.SecureRandom;
 import java.security.Security;
 import java.security.cert.CertificateEncodingException;
 import java.security.cert.CertificateException;
 import java.security.cert.CertificateParsingException;
 import java.security.cert.X509Certificate;
+import java.util.Arrays;
 import java.util.Calendar;
 import java.util.Date;
 import java.util.Random;
@@ -49,6 +51,7 @@ import org.bouncycastle.asn1.x500.X500Name;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.asn1.x509.Extension;
 import org.bouncycastle.asn1.x509.Extensions;
+import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
 import org.bouncycastle.cert.jcajce.JcaX509CertificateHolder;
@@ -83,6 +86,7 @@ public class OcspHelper
     private DEROctetString encodedNonce;
     private X509Certificate ocspResponderCertificate;
     private final JcaX509CertificateConverter certificateConverter = new JcaX509CertificateConverter();
+    private static final Random rand = new SecureRandom();
 
     /**
      * @param checkCertificate Certificate to be OCSP-checked
@@ -104,6 +108,16 @@ public class OcspHelper
         this.ocspUrl = ocspUrl;
     }
 
+    /**
+     * Get the certificate to be OCSP-checked.
+     * 
+     * @return The certificate to be OCSP-checked.
+     */
+    X509Certificate getCertificateToCheck()
+    {
+        return certificateToCheck;
+    }
+
     /**
      * Performs and verifies the OCSP-Request
      *
@@ -122,7 +136,8 @@ public class OcspHelper
     }
 
     /**
-     * Get responder certificate. This is available after {@link #getResponseOcsp()} has been called.
+     * Get responder certificate. This is available after {@link #getResponseOcsp()} has been
+     * called. This method should be used instead of {@code basicResponse.getCerts()[0]}
      *
      * @return The certificate of the responder.
      */
@@ -165,22 +180,14 @@ public class OcspHelper
             else
             {
                 byte[] keyHash = responderID.getKeyHash();
-                //TODO
-                // KeyHash ::= OCTET STRING -- SHA-1 hash of responder's public key
-                //         -- (i.e., the SHA-1 hash of the value of the
-                //         -- BIT STRING subjectPublicKey [excluding
-                //         -- the tag, length, and number of unused
-                //         -- bits] in the responder's certificate)
-                throw new UnsupportedOperationException("search by key hash is not implemented yet");
-
-                // how BC calculates the HeyHash:
-                // see CertificateID.createCertID()
-                //  digCalc is a SHA1DigestCalculator
-                //            SubjectPublicKeyInfo info = issuerCert.getSubjectPublicKeyInfo();
-                //            dgOut = digCalc.getOutputStream();
-                //            dgOut.write(info.getPublicKeyData().getBytes());
-                //            dgOut.close();
-                //            ASN1OctetString issuerKeyHash = new DEROctetString(digCalc.getDigest());
+                if (keyHash != null)
+                {
+                    findResponderCertificateByKeyHash(basicResponse, keyHash);
+                }
+                else
+                {
+                    throw new OCSPException("OCSP: basic response must provide name or key hash");
+                }
             }
 
             if (ocspResponderCertificate == null)
@@ -202,41 +209,110 @@ public class OcspHelper
             boolean nonceChecked = checkNonce(basicResponse);
 
             SingleResp[] responses = basicResponse.getResponses();
-            if (responses.length == 1)
-            {
-                SingleResp resp = responses[0];
-                Object status = resp.getCertStatus();
-
-                if (!nonceChecked)
-                {
-                    // https://tools.ietf.org/html/rfc5019
-                    // fall back to validating the OCSPResponse based on time
-                    checkOcspResponseFresh(resp);
-                }
-
-                if (status instanceof RevokedStatus)
-                {
-                    RevokedStatus revokedStatus = (RevokedStatus) status;
-                    if (revokedStatus.getRevocationTime().compareTo(signDate) <= 0)
-                    {
-                        throw new RevokedCertificateException(
-                            "OCSP: Certificate is revoked since " +
-                                    revokedStatus.getRevocationTime(),
-                                    revokedStatus.getRevocationTime());
-                    }
-                    LOG.info("The certificate was revoked after signing by OCSP " + ocspUrl + 
-                             " on " + revokedStatus.getRevocationTime());
-                }
-                else if (status != CertificateStatus.GOOD)
-                {
-                    throw new OCSPException("OCSP: Status of Cert is unknown");
-                }
-            }
-            else
+            if (responses.length != 1)
             {
                 throw new OCSPException(
                         "OCSP: Received " + responses.length + " responses instead of 1!");
             }
+
+            SingleResp resp = responses[0];
+            Object status = resp.getCertStatus();
+
+            if (!nonceChecked)
+            {
+                // https://tools.ietf.org/html/rfc5019
+                // fall back to validating the OCSPResponse based on time
+                checkOcspResponseFresh(resp);
+            }
+
+            if (status instanceof RevokedStatus)
+            {
+                RevokedStatus revokedStatus = (RevokedStatus) status;
+                if (revokedStatus.getRevocationTime().compareTo(signDate) <= 0)
+                {
+                    throw new RevokedCertificateException(
+                        "OCSP: Certificate is revoked since " +
+                                revokedStatus.getRevocationTime(),
+                                revokedStatus.getRevocationTime());
+                }
+                LOG.info("The certificate was revoked after signing by OCSP " + ocspUrl + 
+                         " on " + revokedStatus.getRevocationTime());
+            }
+            else if (status != CertificateStatus.GOOD)
+            {
+                throw new OCSPException("OCSP: Status of Cert is unknown");
+            }
+        }
+    }
+
+    private byte[] getKeyHashFromCertHolder(X509CertificateHolder certHolder) throws IOException
+    {
+        // https://tools.ietf.org/html/rfc2560#section-4.2.1
+        // KeyHash ::= OCTET STRING -- SHA-1 hash of responder's public key
+        //         -- (i.e., the SHA-1 hash of the value of the
+        //         -- BIT STRING subjectPublicKey [excluding
+        //         -- the tag, length, and number of unused
+        //         -- bits] in the responder's certificate)
+
+        // code below inspired by org.bouncycastle.cert.ocsp.CertificateID.createCertID()
+        // tested with SO52757037-Signed3-OCSP-with-KeyHash.pdf
+        SubjectPublicKeyInfo info = certHolder.getSubjectPublicKeyInfo();
+        try
+        {
+            return MessageDigest.getInstance("SHA-1").digest(info.getPublicKeyData().getBytes());
+        }
+        catch (NoSuchAlgorithmException ex)
+        {
+            // should not happen
+            LOG.error("SHA-1 Algorithm not found", ex);
+            return new byte[0];
+        }
+    }
+
+    private void findResponderCertificateByKeyHash(BasicOCSPResp basicResponse, byte[] keyHash)
+            throws IOException
+    {
+        X509CertificateHolder[] certHolders = basicResponse.getCerts();
+        for (X509CertificateHolder certHolder : certHolders)
+        {
+            byte[] digest = getKeyHashFromCertHolder(certHolder);
+            if (Arrays.equals(keyHash, digest))
+            {
+                try
+                {
+                    ocspResponderCertificate = certificateConverter.getCertificate(certHolder);
+                    return;
+                }
+                catch (CertificateException ex)
+                {
+                    // unlikely to happen because the certificate existed as an object
+                    LOG.error(ex, ex);
+                }
+                break;
+            }
+        }
+
+        // DO NOT use the certificate found in additionalCerts first. One file had a
+        // responder certificate in the PDF itself with SHA1withRSA algorithm, but
+        // the responder delivered a different (newer, more secure) certificate
+        // with SHA256withRSA (tried with QV_RCA1_RCA3_CPCPS_V4_11.pdf)
+        // https://www.quovadisglobal.com/~/media/Files/Repository/QV_RCA1_RCA3_CPCPS_V4_11.ashx
+        for (X509Certificate cert : additionalCerts)
+        {
+            try
+            {
+                byte[] digest = getKeyHashFromCertHolder(new X509CertificateHolder(cert.getEncoded()));
+                if (Arrays.equals(keyHash, digest))
+                {
+                    ocspResponderCertificate = cert;
+                    return;
+                }
+            }
+            catch (CertificateException ex)
+            {
+                // unlikely to happen because the certificate existed as an object
+                LOG.error(ex, ex);
+            }
         }
     }
 
@@ -250,30 +326,28 @@ public class OcspHelper
                 try
                 {
                     ocspResponderCertificate = certificateConverter.getCertificate(certHolder);
+                    return;
                 }
                 catch (CertificateException ex)
                 {
                     // unlikely to happen because the certificate existed as an object
                     LOG.error(ex, ex);
                 }
-                break;
             }
         }
-        if (ocspResponderCertificate == null)
+
+        // DO NOT use the certificate found in additionalCerts first. One file had a
+        // responder certificate in the PDF itself with SHA1withRSA algorithm, but
+        // the responder delivered a different (newer, more secure) certificate
+        // with SHA256withRSA (tried with QV_RCA1_RCA3_CPCPS_V4_11.pdf)
+        // https://www.quovadisglobal.com/~/media/Files/Repository/QV_RCA1_RCA3_CPCPS_V4_11.ashx
+        for (X509Certificate cert : additionalCerts)
         {
-            // DO NOT use the certificate found in additionalCerts first. One file had a
-            // responder certificate in the PDF itself with SHA1withRSA algorithm, but
-            // the responder delivered a different (newer, more secure) certificate
-            // with SHA256withRSA (tried with QV_RCA1_RCA3_CPCPS_V4_11.pdf)
-            // https://www.quovadisglobal.com/~/media/Files/Repository/QV_RCA1_RCA3_CPCPS_V4_11.ashx
-            for (X509Certificate cert : additionalCerts)
+            X500Name certSubjectName = new X500Name(cert.getSubjectX500Principal().getName());
+            if (certSubjectName.equals(name))
             {
-                X500Name certSubjectName = new X500Name(cert.getSubjectX500Principal().getName());
-                if (certSubjectName.equals(name))
-                {
-                    ocspResponderCertificate = cert;
-                    break;
-                }
+                ocspResponderCertificate = cert;
+                return;
             }
         }
     }
@@ -498,10 +572,7 @@ public class OcspHelper
         Extension responseExtension = new Extension(OCSPObjectIdentifiers.id_pkix_ocsp_response,
                 false, new DLSequence(OCSPObjectIdentifiers.id_pkix_ocsp_basic).getEncoded());
 
-        Random rand = new Random();
-        byte[] nonce = new byte[16];
-        rand.nextBytes(nonce);
-        encodedNonce = new DEROctetString(new DEROctetString(nonce));
+        encodedNonce = new DEROctetString(new DEROctetString(create16BytesNonce()));
         Extension nonceExtension = new Extension(OCSPObjectIdentifiers.id_pkix_ocsp_nonce, false,
                 encodedNonce);
 
@@ -512,6 +583,14 @@ public class OcspHelper
         return builder.build();
     }
 
+    private byte[] create16BytesNonce()
+    {
+        // replace with SecureRandom.getInstanceStrong() on jdk8 and higher
+        byte[] nonce = new byte[16];
+        rand.nextBytes(nonce);
+        return nonce;
+    }
+
     /**
      * Class to create SHA-1 Digest, used for creation of CertificateID.
      */
@@ -542,10 +621,11 @@ public class OcspHelper
                 MessageDigest md = MessageDigest.getInstance("SHA-1");
                 return md.digest(bytes);
             }
-            catch (NoSuchAlgorithmException e)
+            catch (NoSuchAlgorithmException ex)
             {
-                LOG.error("SHA-1 Algorithm not found", e);
-                return null;
+                // should not happen
+                LOG.error("SHA-1 Algorithm not found", ex);
+                return new byte[0];
             }
         }
     }
