@@ -23,7 +23,6 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.lang.reflect.InvocationTargetException;
-import java.math.BigInteger;
 import java.security.GeneralSecurityException;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
@@ -65,6 +64,8 @@ import org.bouncycastle.cert.ocsp.OCSPResp;
  * V1.1.2 (2009-12), Part 4: PAdES Long Term - PAdES-LTV Profile. This procedure appends the
  * Validation Information of the last signature (more precise its signer(s)) to a copy of the
  * document. The signature and the signed data will not be touched and stay valid.
+ * <p>
+ * See also <a href="http://eprints.hsr.ch/id/eprint/616">Bachelor thesis (in German) about LTV</a>
  *
  * @author Alexis Suter
  */
@@ -80,8 +81,12 @@ public class AddValidationInformation
     private COSArray crls;
     private COSArray certs;
     private PDDocument document;
-    private final Set<BigInteger> foundRevocationInformation = new HashSet<BigInteger>();
+    private final Set<X509Certificate> foundRevocationInformation = new HashSet<X509Certificate>();
     private Calendar signDate;
+    private final Set<X509Certificate> ocspChecked = new HashSet<X509Certificate>();
+    //TODO foundRevocationInformation and ocspChecked have a similar purpose. One of them should likely
+    // be removed and the code improved. When doing so, keep in mind that ocspChecked was added last,
+    // because of a problem with freetsa.
 
     /**
      * Signs the given PDF file.
@@ -194,33 +199,27 @@ public class AddValidationInformation
             }
             catch (InstantiationException ex)
             {
-                LOG.error("Failed to create new instance of " + clazz.getCanonicalName(), ex);
-                return null;
+                throw new IOException("Failed to create new instance of " + clazz.getCanonicalName(), ex);
             }
             catch (IllegalAccessException ex)
             {
-                LOG.error("Failed to create new instance of " + clazz.getCanonicalName(), ex);
-                return null;
+                throw new IOException("Failed to create new instance of " + clazz.getCanonicalName(), ex);
             }
             catch (NoSuchMethodException ex)
             {
-                LOG.error("Failed to create new instance of " + clazz.getCanonicalName(), ex);
-                return null;
+                throw new IOException("Failed to create new instance of " + clazz.getCanonicalName(), ex);
             }
             catch (SecurityException ex)
             {
-                LOG.error("Failed to create new instance of " + clazz.getCanonicalName(), ex);
-                return null;
+                throw new IOException("Failed to create new instance of " + clazz.getCanonicalName(), ex);
             }
             catch (IllegalArgumentException ex)
             {
-                LOG.error("Failed to create new instance of " + clazz.getCanonicalName(), ex);
-                return null;
+                throw new IOException("Failed to create new instance of " + clazz.getCanonicalName(), ex);
             }
             catch (InvocationTargetException ex)
             {
-                LOG.error("Failed to create new instance of " + clazz.getCanonicalName(), ex);
-                return null;
+                throw new IOException("Failed to create new instance of " + clazz.getCanonicalName(), ex);
             }
             result.setDirect(false);
             parent.setItem(COSName.getPDFName(name), result);
@@ -265,8 +264,7 @@ public class AddValidationInformation
             return;
         }
         // To avoid getting same revocation information twice.
-        boolean isRevocationInfoFound = foundRevocationInformation
-                .contains(certInfo.getCertificate().getSerialNumber());
+        boolean isRevocationInfoFound = foundRevocationInformation.contains(certInfo.getCertificate());
         if (!isRevocationInfoFound)
         {
             if (certInfo.getOcspUrl() != null && certInfo.getIssuerCertificate() != null)
@@ -383,6 +381,11 @@ public class AddValidationInformation
     private void addOcspData(CertSignatureInformation certInfo) throws IOException, OCSPException,
             CertificateProccessingException, RevokedCertificateException
     {
+        if (ocspChecked.contains(certInfo.getCertificate()))
+        {
+            // This certificate has been OCSP-checked before
+            return;
+        }
         OcspHelper ocspHelper = new OcspHelper(
                 certInfo.getCertificate(),
                 signDate.getTime(),
@@ -390,6 +393,7 @@ public class AddValidationInformation
                 new HashSet<X509Certificate>(certInformationHelper.getCertificateSet()),
                 certInfo.getOcspUrl());
         OCSPResp ocspResp = ocspHelper.getResponseOcsp();
+        ocspChecked.add(certInfo.getCertificate());
         BasicOCSPResp basicResponse = (BasicOCSPResp) ocspResp.getResponseObject();
         X509Certificate ocspResponderCertificate = ocspHelper.getOcspResponderCertificate();
         certInformationHelper.addAllCertsFromHolders(basicResponse.getCerts());
@@ -427,7 +431,7 @@ public class AddValidationInformation
         {
             correspondingOCSPs.add(ocspStream);
         }
-        foundRevocationInformation.add(certInfo.getCertificate().getSerialNumber());
+        foundRevocationInformation.add(certInfo.getCertificate());
     }
 
     /**
@@ -498,7 +502,7 @@ public class AddValidationInformation
                 correspondingCRLs = savedCorrespondingCRLs;
             }
         }
-        foundRevocationInformation.add(certInfo.getCertificate().getSerialNumber());
+        foundRevocationInformation.add(certInfo.getCertificate());
     }
 
     private void updateVRI(CertSignatureInformation certInfo, COSDictionary vri) throws IOException
