@@ -22,10 +22,6 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
 import java.security.GeneralSecurityException;
-import java.security.InvalidKeyException;
-import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
-import java.security.SignatureException;
 import java.security.cert.CertificateException;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
@@ -40,8 +36,7 @@ import org.apache.pdfbox.examples.signature.cert.CertificateVerifier;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.encryption.SecurityProvider;
 import org.apache.pdfbox.pdmodel.interactive.digitalsignature.PDSignature;
-import org.bouncycastle.asn1.DERSequence;
-import org.bouncycastle.asn1.DERSet;
+import org.bouncycastle.asn1.ASN1Object;
 import org.bouncycastle.asn1.cms.Attribute;
 import org.bouncycastle.asn1.cms.AttributeTable;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
@@ -51,8 +46,6 @@ import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
 import org.bouncycastle.cms.CMSException;
 import org.bouncycastle.cms.CMSSignedData;
 import org.bouncycastle.cms.SignerInformation;
-import org.bouncycastle.tsp.TSPException;
-import org.bouncycastle.tsp.TimeStampToken;
 import org.bouncycastle.util.Selector;
 import org.bouncycastle.util.Store;
 
@@ -71,6 +64,7 @@ public class CertInformationCollector
     private static final int MAX_CERTIFICATE_CHAIN_DEPTH = 5;
 
     private final Set<X509Certificate> certificateSet = new HashSet<X509Certificate>();
+    private final Set<String> urlSet = new HashSet<String>();
 
     private final JcaX509CertificateConverter certConverter = new JcaX509CertificateConverter();
 
@@ -119,11 +113,7 @@ public class CertInformationCollector
         try
         {
             CMSSignedData signedData = new CMSSignedData(signatureContent);
-            Store<X509CertificateHolder> certificatesStore = signedData.getCertificates();
-
-            SignerInformation signerInformation = processSignerStore(certificatesStore, signedData,
-                    rootCertInfo);
-
+            SignerInformation signerInformation = processSignerStore(signedData, rootCertInfo);
             addTimestampCerts(signerInformation);
         }
         catch (CMSException e)
@@ -152,33 +142,18 @@ public class CertInformationCollector
         }
         Attribute tsAttribute = signerInformation.getUnsignedAttributes()
                 .get(PKCSObjectIdentifiers.id_aa_signatureTimeStampToken);
-        if (tsAttribute.getAttrValues() instanceof DERSet)
+
+        ASN1Object tsSeq = (ASN1Object) tsAttribute.getAttrValues().getObjectAt(0);
+
+        try
         {
-            DERSet tsSet = (DERSet) tsAttribute.getAttrValues();
-            tsSet.getEncoded("DER");
-            DERSequence tsSeq = (DERSequence) tsSet.getObjectAt(0);
-
-            try
-            {
-                TimeStampToken tsToken = new TimeStampToken(
-                        new CMSSignedData(tsSeq.getEncoded("DER")));
-
-                rootCertInfo.tsaCerts = new CertSignatureInformation();
-
-                @SuppressWarnings("unchecked")
-                Store<X509CertificateHolder> certificatesStore = tsToken.getCertificates();
-
-                processSignerStore(certificatesStore, tsToken.toCMSSignedData(),
-                        rootCertInfo.tsaCerts);
-            }
-            catch (TSPException e)
-            {
-                throw new IOException("Error parsing timestamp token", e);
-            }
-            catch (CMSException e)
-            {
-                throw new IOException("Error parsing timestamp token", e);
-            }
+            CMSSignedData signedData = new CMSSignedData(tsSeq.getEncoded("DER"));
+            rootCertInfo.tsaCerts = new CertSignatureInformation();
+            processSignerStore(signedData, rootCertInfo.tsaCerts);
+        }
+        catch (CMSException e)
+        {
+            throw new IOException("Error parsing timestamp token", e);
         }
     }
 
@@ -187,21 +162,21 @@ public class CertInformationCollector
      * to the certInfo. Handles only the first signer, although multiple would be possible, but is
      * not yet practicable.
      *
-     * @param certificatesStore To get the certificate information from. Certificates will be saved
-     * in certificateSet.
      * @param signedData data from which to get the SignerInformation
      * @param certInfo where to add certificate information
      * @return Signer Information of the processed certificatesStore for further usage.
      * @throws IOException on data-processing error
      * @throws CertificateProccessingException on a specific error with a certificate
      */
-    private SignerInformation processSignerStore(Store<X509CertificateHolder> certificatesStore,
+    private SignerInformation processSignerStore(
             CMSSignedData signedData, CertSignatureInformation certInfo)
             throws IOException, CertificateProccessingException
     {
         Collection<SignerInformation> signers = signedData.getSignerInfos().getSigners();
         SignerInformation signerInformation = signers.iterator().next();
 
+        @SuppressWarnings("unchecked")
+        Store<X509CertificateHolder> certificatesStore = signedData.getCertificates();
         @SuppressWarnings("unchecked")
         Collection<X509CertificateHolder> matches = certificatesStore
                 .getMatches((Selector<X509CertificateHolder>) signerInformation.getSID());
@@ -271,23 +246,7 @@ public class CertInformationCollector
                 {
                     certificate.verify(issuer.getPublicKey(), SecurityProvider.getProvider().getName());
                 }
-                catch (CertificateException ex)
-                {
-                    throw new CertificateProccessingException(ex);
-                }
-                catch (NoSuchAlgorithmException ex)
-                {
-                    throw new CertificateProccessingException(ex);
-                }
-                catch (InvalidKeyException ex)
-                {
-                    throw new CertificateProccessingException(ex);
-                }
-                catch (SignatureException ex)
-                {
-                    throw new CertificateProccessingException(ex);
-                }
-                catch (NoSuchProviderException ex)
+                catch (GeneralSecurityException ex)
                 {
                     throw new CertificateProccessingException(ex);
                 }
@@ -319,6 +278,11 @@ public class CertInformationCollector
     private void getAlternativeIssuerCertificate(CertSignatureInformation certInfo, int maxDepth)
             throws CertificateProccessingException
     {
+        if (urlSet.contains(certInfo.issuerUrl))
+        {
+            return;
+        }
+        urlSet.add(certInfo.issuerUrl);
         LOG.info("Get alternative issuer certificate from: " + certInfo.issuerUrl);
         try
         {
