@@ -37,7 +37,6 @@ import org.apache.pdfbox.pdmodel.PDPage;
 import org.apache.pdfbox.pdmodel.common.PDRectangle;
 import org.apache.pdfbox.pdmodel.font.PDCIDFontType2;
 import org.apache.pdfbox.pdmodel.font.PDFont;
-import org.apache.pdfbox.pdmodel.font.PDSimpleFont;
 import org.apache.pdfbox.pdmodel.font.PDTrueTypeFont;
 import org.apache.pdfbox.pdmodel.font.PDType0Font;
 import org.apache.pdfbox.pdmodel.font.PDType3CharProc;
@@ -66,7 +65,6 @@ public class DrawPrintTextLocations extends PDFTextStripper
     private final String filename;
     static final int SCALE = 4;
     private Graphics2D g2d;
-    private final PDDocument document;
 
     /**
      * Instantiate a new PDFTextStripper object.
@@ -77,7 +75,7 @@ public class DrawPrintTextLocations extends PDFTextStripper
      */
     public DrawPrintTextLocations(PDDocument document, String filename) throws IOException
     {
-        this.document = document;
+        this.document = document; // must initialize here, base class initializes too late
         this.filename = filename;
     }
 
@@ -190,15 +188,6 @@ public class DrawPrintTextLocations extends PDFTextStripper
                 }
             }
         }
-        else if (font instanceof PDSimpleFont)
-        {
-            PDSimpleFont simpleFont = (PDSimpleFont) font;
-
-            // these two lines do not always work, e.g. for the TT fonts in file 032431.pdf
-            // which is why PDVectorFont is tried first.
-            String name = simpleFont.getEncoding().getName(code);
-            path = simpleFont.getPath(name);
-        }
         else
         {
             // shouldn't happen, please open issue in JIRA
