@@ -375,6 +375,10 @@ final class Type1Parser
     {
         List<Token> value = new ArrayList<Token>();
         Token token = lexer.nextToken();
+        if (lexer.peekToken() == null)
+        {
+            return value;
+        }
         value.add(token);
 
         if (token.getKind() == Token.START_ARRAY)
@@ -382,6 +386,10 @@ final class Type1Parser
             int openArray = 1;
             while (true)
             {
+                if (lexer.peekToken() == null)
+                {
+                    return value;
+                }
                 if (lexer.peekToken().getKind() == Token.START_ARRAY)
                 {
                     openArray++;
@@ -411,6 +419,12 @@ final class Type1Parser
             return value;
         }
 
+        readPostScriptWrapper(value);
+        return value;
+    }
+
+    private void readPostScriptWrapper(List<Token> value) throws IOException
+    {
         // postscript wrapper (not in the Type 1 spec)
         if (lexer.peekToken().getText().equals("systemdict"))
         {
@@ -435,7 +449,6 @@ final class Type1Parser
 
             read(Token.NAME, "if");
         }
-        return value;
     }
 
     /**
@@ -834,7 +847,7 @@ final class Type1Parser
     private Token readMaybe(Token.Kind kind, String name) throws IOException
     {
         Token token = lexer.peekToken();
-        if (token.getKind() == kind && token.getText().equals(name))
+        if (token != null && token.getKind() == kind && token.getText().equals(name))
         {
             return lexer.nextToken();
         }
