@@ -105,6 +105,8 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
                 optionUncompressed = (options & TIFFExtension.GROUP4OPT_UNCOMPRESSED) != 0;
                 optionByteAligned = (options & TIFFExtension.GROUP4OPT_BYTEALIGNED) != 0;
                 break;
+            default:
+                break;
         }
 
     }
@@ -303,6 +305,8 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
             case TIFFExtension.COMPRESSION_CCITT_T6:
                 decodeRowType6();
                 break;
+            default:
+                break;
         }
 
         int index = 0;
@@ -382,7 +386,8 @@ final class CCITTFaxDecoderStream extends FilterInputStream {
         }
     }
 
-    private void resetBuffer() throws IOException {
+    private void resetBuffer()
+    {
         bufferPos = -1;
     }
 
