@@ -16,8 +16,6 @@
  */
 package org.apache.pdfbox.filter;
 
-import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -28,7 +26,6 @@ import java.util.zip.Inflater;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSDictionary;
-import org.apache.pdfbox.cos.COSName;
 
 /**
  * Decompresses data encoded using the zlib/deflate compression method,
@@ -46,34 +43,12 @@ final class FlateFilter extends Filter
     public DecodeResult decode(InputStream encoded, OutputStream decoded,
                                          COSDictionary parameters, int index) throws IOException
     {
-        int predictor = -1;
-
         final COSDictionary decodeParams = getDecodeParams(parameters, index);
-        if (decodeParams != null)
-        {
-            predictor = decodeParams.getInt(COSName.PREDICTOR);
-        }
 
         try
         {
-            if (predictor > 1)
-            {
-                int colors = Math.min(decodeParams.getInt(COSName.COLORS, 1), 32);
-                int bitsPerPixel = decodeParams.getInt(COSName.BITS_PER_COMPONENT, 8);
-                int columns = decodeParams.getInt(COSName.COLUMNS, 1);
-                ByteArrayOutputStream baos = new ByteArrayOutputStream();
-                decompress(encoded, baos);
-                ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
-                Predictor.decodePredictor(predictor, colors, bitsPerPixel, columns, bais, decoded);
-                decoded.flush();
-                baos.reset();
-                bais.reset();
-            }
-            else
-            {
-                decompress(encoded, decoded);
-            }
-        } 
+            decompress(encoded, Predictor.wrapPredictor(decoded, decodeParams));
+        }
         catch (DataFormatException e)
         {
             // if the stream is corrupt a DataFormatException may occur
@@ -98,7 +73,7 @@ final class FlateFilter extends Filter
             // use nowrap mode to bypass zlib-header and checksum to avoid a DataFormatException
             Inflater inflater = new Inflater(true); 
             inflater.setInput(buf,0,read);
-            byte[] res = new byte[1024]; 
+            byte[] res = new byte[1024];
             boolean dataWritten = false;
             while (true) 
             { 
@@ -143,16 +118,7 @@ final class FlateFilter extends Filter
     protected void encode(InputStream input, OutputStream encoded, COSDictionary parameters)
             throws IOException
     {
-        int compressionLevel = Deflater.DEFAULT_COMPRESSION;
-        try
-        {
-            compressionLevel = Integer.parseInt(System.getProperty(Filter.SYSPROP_DEFLATELEVEL, "-1"));
-        }
-        catch (NumberFormatException ex)
-        {
-            LOG.warn(ex.getMessage(), ex);
-        }
-        compressionLevel = Math.max(-1, Math.min(Deflater.BEST_COMPRESSION, compressionLevel));
+        int compressionLevel = getCompressionLevel();
         Deflater deflater = new Deflater(compressionLevel);
         DeflaterOutputStream out = new DeflaterOutputStream(encoded, deflater);
         int amountRead;
@@ -167,5 +133,6 @@ final class FlateFilter extends Filter
         }
         out.close();
         encoded.flush();
+        deflater.end();
     }
 }
