@@ -30,6 +30,7 @@ import java.text.NumberFormat;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Deque;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Hashtable;
 import java.util.Iterator;
@@ -182,8 +183,10 @@ public class COSWriter implements ICOSVisitor, Closeable
     // these are used for indirect references in other objects
     //A hashtable is used on purpose over a hashmap
     //so that null entries will not get added.
+    @SuppressWarnings({"squid:S1149"})
     private final Map<COSBase,COSObjectKey> objectKeys = new Hashtable<COSBase,COSObjectKey>();
-    private final Map<COSObjectKey,COSBase> keyObject = new Hashtable<COSObjectKey,COSBase>();
+
+    private final Map<COSObjectKey,COSBase> keyObject = new HashMap<COSObjectKey,COSBase>();
 
     // the list of x ref entries to be made so far
     private final List<COSWriterXRefEntry> xRefEntries = new ArrayList<COSWriterXRefEntry>();
@@ -211,8 +214,10 @@ public class COSWriter implements ICOSVisitor, Closeable
     // signing
     private boolean incrementalUpdate = false;
     private boolean reachedSignature = false;
-    private long signatureOffset, signatureLength;
-    private long byteRangeOffset, byteRangeLength;
+    private long signatureOffset;
+    private long signatureLength;
+    private long byteRangeOffset;
+    private long byteRangeLength;
     private RandomAccessRead incrementalInput;
     private OutputStream incrementalOutput;
     private SignatureInterface signatureInterface;
@@ -786,7 +791,7 @@ public class COSWriter implements ICOSVisitor, Closeable
         }
         byte[] signatureBytes = Hex.getBytes(cmsSignature);
 
-        // substract 2 bytes because of the enclosing "<>"
+        // subtract 2 bytes because of the enclosing "<>"
         if (signatureBytes.length > signatureLength - 2)
         {
             throw new IOException("Can't write signature, not enough space");
