@@ -44,6 +44,7 @@ import org.apache.pdfbox.cos.COSInteger;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSNumber;
 import org.apache.pdfbox.cos.COSObject;
+import org.apache.pdfbox.cos.COSUpdateInfo;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.io.MemoryUsageSetting;
 import org.apache.pdfbox.io.RandomAccessBuffer;
@@ -66,6 +67,7 @@ import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceRGB;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotation;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotationWidget;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAppearanceDictionary;
+import org.apache.pdfbox.pdmodel.interactive.annotation.PDAppearanceStream;
 import org.apache.pdfbox.pdmodel.interactive.digitalsignature.ExternalSigningSupport;
 import org.apache.pdfbox.pdmodel.interactive.digitalsignature.PDSignature;
 import org.apache.pdfbox.pdmodel.interactive.digitalsignature.SignatureInterface;
@@ -217,6 +219,42 @@ public class PDDocument implements Closeable
         pages.setItem(COSName.COUNT, COSInteger.ZERO);
     }
 
+    /**
+     * Constructor that uses an existing document. The COSDocument that is passed in must be valid.
+     * 
+     * @param doc The COSDocument that this document wraps.
+     */
+    public PDDocument(COSDocument doc)
+    {
+        this(doc, null);
+    }
+
+    /**
+     * Constructor that uses an existing document. The COSDocument that is passed in must be valid.
+     * 
+     * @param doc The COSDocument that this document wraps.
+     * @param source the parser which is used to read the pdf
+     */
+    public PDDocument(COSDocument doc, RandomAccessRead source)
+    {
+        this(doc, source, null);
+    }
+
+    /**
+     * Constructor that uses an existing document. The COSDocument that is passed in must be valid.
+     * 
+     * @param doc The COSDocument that this document wraps.
+     * @param source the parser which is used to read the pdf
+     * @param permission he access permissions of the pdf
+     * 
+     */
+    public PDDocument(COSDocument doc, RandomAccessRead source, AccessPermission permission)
+    {
+        document = doc;
+        pdfSource = source;
+        accessPermission = permission;
+    }
+
     /**
      * This will add a page to the document. This is a convenience method, that will add the page to the root of the
      * hierarchy and set the parent of the page to the root.
@@ -603,12 +641,18 @@ public class PDDocument implements Closeable
     }
 
     private void prepareNonVisibleSignature(PDSignatureField signatureField)
-            throws IOException
     {
         // "Signature fields that are not intended to be visible shall
         // have an annotation rectangle that has zero height and width."
         // Set rectangle for non-visual signature to rectangle array [ 0 0 0 0 ]
         signatureField.getWidgets().get(0).setRectangle(new PDRectangle());
+        
+        // The visual appearance must also exist for an invisible signature but may be empty.
+        PDAppearanceDictionary appearanceDictionary = new PDAppearanceDictionary();
+        PDAppearanceStream appearanceStream = new PDAppearanceStream(this);
+        appearanceStream.setBBox(new PDRectangle());
+        appearanceDictionary.setNormalAppearance(appearanceStream);
+        signatureField.getWidgets().get(0).setAppearance(appearanceDictionary);
     }
 
     /**
@@ -736,42 +780,6 @@ public class PDDocument implements Closeable
         return importedPage;
     }
 
-    /**
-     * Constructor that uses an existing document. The COSDocument that is passed in must be valid.
-     * 
-     * @param doc The COSDocument that this document wraps.
-     */
-    public PDDocument(COSDocument doc)
-    {
-        this(doc, null);
-    }
-
-    /**
-     * Constructor that uses an existing document. The COSDocument that is passed in must be valid.
-     * 
-     * @param doc The COSDocument that this document wraps.
-     * @param source the parser which is used to read the pdf
-     */
-    public PDDocument(COSDocument doc, RandomAccessRead source)
-    {
-        this(doc, source, null);
-    }
-
-    /**
-     * Constructor that uses an existing document. The COSDocument that is passed in must be valid.
-     * 
-     * @param doc The COSDocument that this document wraps.
-     * @param source the parser which is used to read the pdf
-     * @param permission he access permissions of the pdf
-     * 
-     */
-    public PDDocument(COSDocument doc, RandomAccessRead source, AccessPermission permission)
-    {
-        document = doc;
-        pdfSource = source;
-        accessPermission = permission;
-    }
-
     /**
      * This will get the low level document.
      * 
@@ -783,16 +791,21 @@ public class PDDocument implements Closeable
     }
 
     /**
-     * This will get the document info dictionary. This is guaranteed to not return null.
-     * 
-     * @return The documents /Info dictionary
+     * This will get the document info dictionary. If it doesn't exist, an empty document info
+     * dictionary is created in the document trailer.
+     * <p>
+     * In PDF 2.0 this is deprecated except for two entries, /CreationDate and /ModDate. For any other
+     * document level metadata, a metadata stream should be used instead, see
+     * {@link PDDocumentCatalog#getMetadata()}.
+     *
+     * @return The documents /Info dictionary, never null.
      */
     public PDDocumentInformation getDocumentInformation()
     {
         if (documentInformation == null)
         {
             COSDictionary trailer = document.getTrailer();
-            COSDictionary infoDic = (COSDictionary) trailer.getDictionaryObject(COSName.INFO);
+            COSDictionary infoDic = trailer.getCOSDictionary(COSName.INFO);
             if (infoDic == null)
             {
                 infoDic = new COSDictionary();
@@ -805,7 +818,11 @@ public class PDDocument implements Closeable
 
     /**
      * This will set the document information for this document.
-     * 
+     * <p>
+     * In PDF 2.0 this is deprecated except for two entries, /CreationDate and /ModDate. For any other
+     * document level metadata, a metadata stream should be used instead, see
+     * {@link PDDocumentCatalog#setMetadata(org.apache.pdfbox.pdmodel.common.PDMetadata) PDDocumentCatalog#setMetadata(PDMetadata)}.
+     *
      * @param info The updated document information.
      */
     public void setDocumentInformation(PDDocumentInformation info)
@@ -849,7 +866,7 @@ public class PDDocument implements Closeable
 
     /**
      * This will get the encryption dictionary for this document. This will still return the parameters if the document
-     * was decrypted. As the encryption architecture in PDF documents is plugable this returns an abstract class,
+     * was decrypted. As the encryption architecture in PDF documents is pluggable this returns an abstract class,
      * but the only supported subclass at this time is a
      * PDStandardEncryption object.
      *
@@ -967,7 +984,7 @@ public class PDDocument implements Closeable
      * @throws InvalidPasswordException If the file required a non-empty password.
      * @throws IOException in case of a file reading or parsing error
      */
-    public static PDDocument load(File file) throws InvalidPasswordException, IOException
+    public static PDDocument load(File file) throws IOException
     {
         return load(file, "", MemoryUsageSetting.setupMainMemoryOnly());
     }
@@ -984,7 +1001,7 @@ public class PDDocument implements Closeable
      * @throws IOException in case of a file reading or parsing error
      */
     public static PDDocument load(File file, MemoryUsageSetting memUsageSetting)
-            throws InvalidPasswordException, IOException
+            throws IOException
     {
         return load(file, "", null, null, memUsageSetting);
     }
@@ -1001,7 +1018,7 @@ public class PDDocument implements Closeable
      * @throws IOException in case of a file reading or parsing error
      */
     public static PDDocument load(File file, String password)
-            throws InvalidPasswordException, IOException
+            throws IOException
     {
         return load(file, password, null, null, MemoryUsageSetting.setupMainMemoryOnly());
     }
@@ -1019,7 +1036,7 @@ public class PDDocument implements Closeable
      * @throws IOException in case of a file reading or parsing error
      */
     public static PDDocument load(File file, String password, MemoryUsageSetting memUsageSetting)
-            throws InvalidPasswordException, IOException
+            throws IOException
     {
         return load(file, password, null, null, memUsageSetting);
     }
@@ -1058,21 +1075,11 @@ public class PDDocument implements Closeable
     public static PDDocument load(File file, String password, InputStream keyStore, String alias,
                                   MemoryUsageSetting memUsageSetting) throws IOException
     {
+        @SuppressWarnings({"squid:S2095"}) // raFile not closed here, may be needed for signing
         RandomAccessBufferedFileInputStream raFile = new RandomAccessBufferedFileInputStream(file);
         try
         {
-            ScratchFile scratchFile = new ScratchFile(memUsageSetting);
-            try
-            {
-                PDFParser parser = new PDFParser(raFile, password, keyStore, alias, scratchFile);
-                parser.parse();
-                return parser.getPDDocument();
-            }
-            catch (IOException ioe)
-            {
-                IOUtils.closeQuietly(scratchFile);
-                throw ioe;
-            }
+            return load(raFile, password, keyStore, alias, memUsageSetting);
         }
         catch (IOException ioe)
         {
@@ -1081,28 +1088,45 @@ public class PDDocument implements Closeable
         }
     }
 
+    private static PDDocument load(RandomAccessBufferedFileInputStream raFile, String password,
+                                   InputStream keyStore, String alias,
+                                   MemoryUsageSetting memUsageSetting) throws IOException
+    {
+        ScratchFile scratchFile = new ScratchFile(memUsageSetting);
+        try
+        {
+            PDFParser parser = new PDFParser(raFile, password, keyStore, alias, scratchFile);
+            parser.parse();
+            return parser.getPDDocument();
+        }
+        catch (IOException ioe)
+        {
+            IOUtils.closeQuietly(scratchFile);
+            throw ioe;
+        }
+    }
+
     /**
-     * Parses a PDF. The given input stream is copied to the memory to enable random access to the pdf.
-     * Unrestricted main memory will be used for buffering PDF streams.
+     * Parses a PDF. The given input stream is copied to the memory to enable random access to the
+     * pdf. Unrestricted main memory will be used for buffering PDF streams.
      * 
-     * @param input stream that contains the document.
+     * @param input stream that contains the document. Don't forget to close it after loading.
      * 
      * @return loaded document
      * 
      * @throws InvalidPasswordException If the PDF required a non-empty password.
      * @throws IOException In case of a reading or parsing error.
      */
-    public static PDDocument load(InputStream input) throws InvalidPasswordException, IOException
+    public static PDDocument load(InputStream input) throws IOException
     {
         return load(input, "", null, null, MemoryUsageSetting.setupMainMemoryOnly());
     }
 
     /**
-     * Parses a PDF. Depending on the memory settings parameter the given input
-     * stream is either copied to main memory or to a temporary file to enable
-     * random access to the pdf.
+     * Parses a PDF. Depending on the memory settings parameter the given input stream is either
+     * copied to main memory or to a temporary file to enable random access to the pdf.
      * 
-     * @param input stream that contains the document.
+     * @param input stream that contains the document. Don't forget to close it after loading.
      * @param memUsageSetting defines how memory is used for buffering input stream and PDF streams 
      * 
      * @return loaded document
@@ -1111,34 +1135,34 @@ public class PDDocument implements Closeable
      * @throws IOException In case of a reading or parsing error.
      */
     public static PDDocument load(InputStream input, MemoryUsageSetting memUsageSetting)
-            throws InvalidPasswordException, IOException
+            throws IOException
     {
         return load(input, "", null, null, memUsageSetting);
     }
 
     /**
-     * Parses a PDF. The given input stream is copied to the memory to enable random access to the pdf.
-     * Unrestricted main memory will be used for buffering PDF streams.
-     * 
-     * @param input stream that contains the document.
+     * Parses a PDF. The given input stream is copied to the memory to enable random access to the
+     * pdf. Unrestricted main memory will be used for buffering PDF streams.
+     *
+     * @param input stream that contains the document. Don't forget to close it after loading.
      * @param password password to be used for decryption
-     * 
+     *
      * @return loaded document
      * 
      * @throws InvalidPasswordException If the password is incorrect.
      * @throws IOException In case of a reading or parsing error.
      */
     public static PDDocument load(InputStream input, String password)
-            throws InvalidPasswordException, IOException
+            throws IOException
     {
         return load(input, password, null, null, MemoryUsageSetting.setupMainMemoryOnly());
     }
 
     /**
-     * Parses a PDF. The given input stream is copied to the memory to enable random access to the pdf.
-     * Unrestricted main memory will be used for buffering PDF streams.
-     * 
-     * @param input stream that contains the document.
+     * Parses a PDF. The given input stream is copied to the memory to enable random access to the
+     * pdf. Unrestricted main memory will be used for buffering PDF streams.
+     *
+     * @param input stream that contains the document. Don't forget to close it after loading.
      * @param password password to be used for decryption
      * @param keyStore key store to be used for decryption when using public key security 
      * @param alias alias to be used for decryption when using public key security
@@ -1154,11 +1178,10 @@ public class PDDocument implements Closeable
     }
 
     /**
-     * Parses a PDF. Depending on the memory settings parameter the given input
-     * stream is either copied to main memory or to a temporary file to enable
-     * random access to the pdf.
-     * 
-     * @param input stream that contains the document.
+     * Parses a PDF. Depending on the memory settings parameter the given input stream is either
+     * copied to main memory or to a temporary file to enable random access to the pdf.
+     *
+     * @param input stream that contains the document. Don't forget to close it after loading.
      * @param password password to be used for decryption
      * @param memUsageSetting defines how memory is used for buffering input stream and PDF streams 
      * 
@@ -1168,17 +1191,16 @@ public class PDDocument implements Closeable
      * @throws IOException In case of a reading or parsing error.
      */
     public static PDDocument load(InputStream input, String password, MemoryUsageSetting memUsageSetting)
-            throws InvalidPasswordException, IOException
+            throws IOException
     {
         return load(input, password, null, null, memUsageSetting);
     }
     
     /**
-     * Parses a PDF. Depending on the memory settings parameter the given input
-     * stream is either copied to memory or to a temporary file to enable
-     * random access to the pdf.
-     * 
-     * @param input stream that contains the document.
+     * Parses a PDF. Depending on the memory settings parameter the given input stream is either
+     * copied to memory or to a temporary file to enable random access to the pdf.
+     *
+     * @param input stream that contains the document. Don't forget to close it after loading.
      * @param password password to be used for decryption
      * @param keyStore key store to be used for decryption when using public key security 
      * @param alias alias to be used for decryption when using public key security
@@ -1217,7 +1239,7 @@ public class PDDocument implements Closeable
      * @throws InvalidPasswordException If the PDF required a non-empty password.
      * @throws IOException In case of a reading or parsing error.
      */
-    public static PDDocument load(byte[] input) throws InvalidPasswordException, IOException
+    public static PDDocument load(byte[] input) throws IOException
     {
         return load(input, "");
     }
@@ -1234,7 +1256,7 @@ public class PDDocument implements Closeable
      * @throws IOException In case of a reading or parsing error.
      */
     public static PDDocument load(byte[] input, String password)
-            throws InvalidPasswordException, IOException
+            throws IOException
     {
         return load(input, password, null, null);
     }
@@ -1342,7 +1364,13 @@ public class PDDocument implements Closeable
 
     /**
      * Save the PDF as an incremental update. This is only possible if the PDF was loaded from a
-     * file or a stream, not if the document was created in PDFBox itself.
+     * file or a stream, not if the document was created in PDFBox itself. There must be a path of
+     * objects that have {@link COSUpdateInfo#isNeedToBeUpdated()} set, starting from the document
+     * catalog. For signatures this is taken care by PDFBox itself.
+     * <p>
+     * Other usages of this method are for experienced users only. You will usually never need it.
+     * It is useful only if you are required to keep the current revision and append the changes. A
+     * typical use case is changing a signed file without invalidating the signature.
      *
      * @param output stream to write to. It will be closed when done. It
      * <i><b>must never</b></i> point to the source file or that one will be
@@ -1350,6 +1378,7 @@ public class PDDocument implements Closeable
      * @throws IOException if the output could not be written
      * @throws IllegalStateException if the document was not loaded from a file or a stream.
      */
+    
     public void saveIncremental(OutputStream output) throws IOException
     {
         COSWriter writer = null;
@@ -1487,7 +1516,7 @@ public class PDDocument implements Closeable
             // Make sure that:
             // - first Exception is kept
             // - all IO resources are closed
-            // - there's a way to see which errors occured
+            // - there's a way to see which errors occurred
 
             IOException firstException = null;
 
