@@ -177,9 +177,7 @@ public class PDPage implements COSObjectable, PDContentStream
     }
 
     /**
-     * Returns true if this page has contents.
-     * 
-     * @return true if the page has contents.
+     * Returns true if this page has one or more content streams.
      */
     public boolean hasContents()
     {
@@ -235,11 +233,12 @@ public class PDPage implements COSObjectable, PDContentStream
     /**
      * This will get the key of this Page in the structural parent tree.
      * 
-     * @return the integer key of the page's entry in the structural parent tree
+     * @return the integer key of the page's entry in the structural parent tree or -1 if
+     * there isn't any.
      */
     public int getStructParents()
     {
-        return page.getInt(COSName.STRUCT_PARENTS, 0);
+        return page.getInt(COSName.STRUCT_PARENTS);
     }
 
     /**
@@ -785,4 +784,33 @@ public class PDPage implements COSObjectable, PDContentStream
         }
         page.setItem(COSName.VP, array);
     }
+
+    /**
+     * Get the user unit. This is a positive number that shall give the size of default user space
+     * units, in multiples of 1/72 inch, or 1 if it hasn't been set. This is supported by PDF 1.6
+     * and higher.
+     *
+     * @return the user unit.
+     */
+    public float getUserUnit()
+    {
+        float userUnit = page.getFloat(COSName.USER_UNIT, 1.0f);
+        return userUnit > 0 ? userUnit : 1.0f;
+    }
+
+    /**
+     * Get the user unit. This is a positive number that shall give the size of default user space
+     * units, in multiples of 1/72 inch. This is supported by PDF 1.6 and higher.
+     *
+     * @param userUnit
+     * throws IllegalArgumentException if the parameter is not positive.
+     */
+    public void setUserUnit(float userUnit)
+    {
+        if (userUnit <= 0)
+        {
+            throw new IllegalArgumentException("User unit must be positive");
+        }
+        page.setFloat(COSName.USER_UNIT, userUnit);
+    }
 }
