@@ -27,6 +27,7 @@ import java.util.Locale;
 import java.util.Stack;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.apache.pdfbox.contentstream.operator.OperatorName;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSName;
@@ -110,6 +111,8 @@ public final class PDPageContentStream implements Closeable
     private final NumberFormat formatDecimal = NumberFormat.getNumberInstance(Locale.US);
     private final byte[] formatBuffer = new byte[32];
 
+    private boolean sourcePageHadContents = false;
+
     /**
      * Create a new PDPage content stream. This constructor overwrites all existing content streams
      * of this page.
@@ -121,6 +124,10 @@ public final class PDPageContentStream implements Closeable
     public PDPageContentStream(PDDocument document, PDPage sourcePage) throws IOException
     {
         this(document, sourcePage, AppendMode.OVERWRITE, true, false);
+        if (sourcePageHadContents)
+        {
+            LOG.warn("You are overwriting an existing content, you should use the append mode");
+        }
     }
 
     /**
@@ -255,10 +262,7 @@ public final class PDPageContentStream implements Closeable
         }
         else
         {
-            if (sourcePage.hasContents())
-            {
-                LOG.warn("You are overwriting an existing content, you should use the append mode");
-            }
+            sourcePageHadContents = sourcePage.hasContents();
             PDStream contents = new PDStream(document);
             sourcePage.setContents(contents);
             output = contents.createOutputStream(filter);
@@ -363,7 +367,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: Nested beginText() calls are not allowed.");
         }
-        writeOperator("BT");
+        writeOperator(OperatorName.BEGIN_TEXT);
         inTextMode = true;
     }
 
@@ -380,7 +384,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: You must call beginText() before calling endText.");
         }
-        writeOperator("ET");
+        writeOperator(OperatorName.END_TEXT);
         inTextMode = false;
     }
     
@@ -409,7 +413,7 @@ public final class PDPageContentStream implements Closeable
         
         writeOperand(resources.add(font));
         writeOperand(fontSize);
-        writeOperator("Tf");
+        writeOperator(OperatorName.SET_FONT_AND_SIZE);
     }
 
     /**
@@ -457,7 +461,7 @@ public final class PDPageContentStream implements Closeable
             }
         }
         write("] ");
-        writeOperator("TJ");
+        writeOperator(OperatorName.SHOW_TEXT_ADJUSTED);
     }
 
     /**
@@ -471,7 +475,7 @@ public final class PDPageContentStream implements Closeable
     {
         showTextInternal(text);
         write(" ");
-        writeOperator("Tj");
+        writeOperator(OperatorName.SHOW_TEXT);
     }
 
     /**
@@ -532,7 +536,7 @@ public final class PDPageContentStream implements Closeable
     public void setLeading(float leading) throws IOException
     {
         writeOperand(leading);
-        writeOperator("TL");
+        writeOperator(OperatorName.SET_TEXT_LEADING);
     }
 
     /**
@@ -547,7 +551,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Must call beginText() before newLine()");
         }
-        writeOperator("T*");
+        writeOperator(OperatorName.NEXT_LINE);
     }
 
     /**
@@ -581,7 +585,7 @@ public final class PDPageContentStream implements Closeable
         }
         writeOperand(tx);
         writeOperand(ty);
-        writeOperator("Td");
+        writeOperator(OperatorName.MOVE_TEXT);
     }
 
     /**
@@ -630,7 +634,7 @@ public final class PDPageContentStream implements Closeable
             throw new IllegalStateException("Error: must call beginText() before setTextMatrix");
         }
         writeAffineTransform(matrix.createAffineTransform());
-        writeOperator("Tm");
+        writeOperator(OperatorName.SET_MATRIX);
     }
 
     /**
@@ -717,7 +721,7 @@ public final class PDPageContentStream implements Closeable
         transform(new Matrix(transform));
 
         writeOperand(resources.add(image));
-        writeOperator("Do");
+        writeOperator(OperatorName.DRAW_OBJECT);
 
         restoreGraphicsState();
     }
@@ -744,7 +748,7 @@ public final class PDPageContentStream implements Closeable
         transform(new Matrix(transform));
 
         writeOperand(resources.add(image));
-        writeOperator("Do");
+        writeOperator(OperatorName.DRAW_OBJECT);
 
         restoreGraphicsState();
     }
@@ -821,7 +825,7 @@ public final class PDPageContentStream implements Closeable
 
         // create the image dictionary
         StringBuilder sb = new StringBuilder();
-        sb.append("BI");
+        sb.append(OperatorName.BEGIN_INLINE_IMAGE);
 
         sb.append("\n /W ");
         sb.append(inlineImage.getWidth());
@@ -858,10 +862,10 @@ public final class PDPageContentStream implements Closeable
         writeLine();
 
         // binary data
-        writeOperator("ID");
+        writeOperator(OperatorName.BEGIN_INLINE_IMAGE_DATA);
         writeBytes(inlineImage.getData());
         writeLine();
-        writeOperator("EI");
+        writeOperator(OperatorName.END_INLINE_IMAGE);
 
         restoreGraphicsState();
     }
@@ -920,7 +924,7 @@ public final class PDPageContentStream implements Closeable
         transform(new Matrix(transform));
 
         writeOperand(objMapping);
-        writeOperator("Do");
+        writeOperator(OperatorName.DRAW_OBJECT);
 
         restoreGraphicsState();
     }
@@ -940,7 +944,7 @@ public final class PDPageContentStream implements Closeable
         }
 
         writeOperand(resources.add(form));
-        writeOperator("Do");
+        writeOperator(OperatorName.DRAW_OBJECT);
     }
 
     /**
@@ -974,7 +978,10 @@ public final class PDPageContentStream implements Closeable
     }
 
     /**
-     * The cm operator. Concatenates the given matrix with the CTM.
+     * The cm operator. Concatenates the given matrix with the current transformation matrix (CTM),
+     * which maps user space coordinates used within a PDF content stream into output device
+     * coordinates. More details on coordinates can be found in the PDF 32000 specification, 8.3.2
+     * Coordinate Spaces.
      *
      * @param matrix the transformation matrix
      * @throws IOException If there is an error writing to the stream.
@@ -987,7 +994,7 @@ public final class PDPageContentStream implements Closeable
         }
 
         writeAffineTransform(matrix.createAffineTransform());
-        writeOperator("cm");
+        writeOperator(OperatorName.CONCAT);
     }
 
     /**
@@ -1013,7 +1020,7 @@ public final class PDPageContentStream implements Closeable
         {
             nonStrokingColorSpaceStack.push(nonStrokingColorSpaceStack.peek());
         }
-        writeOperator("q");
+        writeOperator(OperatorName.SAVE);
     }
 
     /**
@@ -1039,7 +1046,7 @@ public final class PDPageContentStream implements Closeable
         {
             nonStrokingColorSpaceStack.pop();
         }
-        writeOperator("Q");
+        writeOperator(OperatorName.RESTORE);
     }
 
     /**
@@ -1055,7 +1062,7 @@ public final class PDPageContentStream implements Closeable
     {
         setStrokingColorSpaceStack(colorSpace);
         writeOperand(getName(colorSpace));
-        writeOperator("CS");
+        writeOperator(OperatorName.STROKING_COLORSPACE);
     }
 
     /**
@@ -1071,7 +1078,7 @@ public final class PDPageContentStream implements Closeable
     {
         setNonStrokingColorSpaceStack(colorSpace);
         writeOperand(getName(colorSpace));
-        writeOperator("cs");
+        writeOperator(OperatorName.NON_STROKING_COLORSPACE);
     }
 
     private COSName getName(PDColorSpace colorSpace) throws IOException
@@ -1100,7 +1107,7 @@ public final class PDPageContentStream implements Closeable
             strokingColorSpaceStack.peek() != color.getColorSpace())
         {
             writeOperand(getName(color.getColorSpace()));
-            writeOperator("CS");
+            writeOperator(OperatorName.STROKING_COLORSPACE);
             setStrokingColorSpaceStack(color.getColorSpace());
         }
 
@@ -1119,11 +1126,11 @@ public final class PDPageContentStream implements Closeable
             color.getColorSpace() instanceof PDDeviceN ||
             color.getColorSpace() instanceof PDICCBased)
         {
-            writeOperator("SCN");
+            writeOperator(OperatorName.STROKING_COLOR_N);
         }
         else
         {
-            writeOperator("SC");
+            writeOperator(OperatorName.STROKING_COLOR);
         }
     }
 
@@ -1167,14 +1174,37 @@ public final class PDPageContentStream implements Closeable
             currentStrokingColorSpace instanceof PDPattern ||
             currentStrokingColorSpace instanceof PDICCBased)
         {
-            writeOperator("SCN");
+            writeOperator(OperatorName.STROKING_COLOR_N);
         }
         else
         {
-            writeOperator("SC");
+            writeOperator(OperatorName.STROKING_COLOR);
         }
     }
 
+    /**
+     * Set the stroking color in the DeviceRGB color space. Range is 0..1.
+     *
+     * @param r The red value
+     * @param g The green value.
+     * @param b The blue value.
+     * @throws IOException If an IO error occurs while writing to the stream.
+     * @throws IllegalArgumentException If the parameters are invalid.
+     */
+    public void setStrokingColor(float r, float g, float b) throws IOException
+    {
+        if (isOutsideOneInterval(r) || isOutsideOneInterval(g) || isOutsideOneInterval(b))
+        {
+            throw new IllegalArgumentException("Parameters must be within 0..1, but are "
+                    + String.format("(%.2f,%.2f,%.2f)", r, g, b));
+        }
+        writeOperand(r);
+        writeOperand(g);
+        writeOperand(b);
+        writeOperator(OperatorName.STROKING_COLOR_RGB);
+        setStrokingColorSpaceStack(PDDeviceRGB.INSTANCE);
+    }
+
     /**
      * Set the stroking color in the DeviceRGB color space. Range is 0..255.
      *
@@ -1183,7 +1213,10 @@ public final class PDPageContentStream implements Closeable
      * @param b The blue value.
      * @throws IOException If an IO error occurs while writing to the stream.
      * @throws IllegalArgumentException If the parameters are invalid.
+     * @deprecated use
+     * {@link #setStrokingColor(float, float, float) setStrokingColor(r/255f, g/255f, b/255f)}
      */
+    @Deprecated
     public void setStrokingColor(int r, int g, int b) throws IOException
     {
         if (isOutside255Interval(r) || isOutside255Interval(g) || isOutside255Interval(b))
@@ -1191,11 +1224,7 @@ public final class PDPageContentStream implements Closeable
             throw new IllegalArgumentException("Parameters must be within 0..255, but are "
                     + String.format("(%d,%d,%d)", r, g, b));
         }
-        writeOperand(r / 255f);
-        writeOperand(g / 255f);
-        writeOperand(b / 255f);
-        writeOperator("RG");
-        setStrokingColorSpaceStack(PDDeviceRGB.INSTANCE);
+        setStrokingColor(r / 255f, g / 255f, b / 255f);
     }
 
     /**
@@ -1207,7 +1236,7 @@ public final class PDPageContentStream implements Closeable
      * @param k The black value.
      * @throws IOException If an IO error occurs while writing to the stream.
      * @throws IllegalArgumentException If the parameters are invalid.
-     * @deprecated Use {@link #setStrokingColor(float, float, float, float)} instead.
+     * @deprecated Use {@link #setStrokingColor(float, float, float, float) setStrokingColor(c/255f, m/255f, y/255f, k/255f)} instead.
      */
     @Deprecated
     public void setStrokingColor(int c, int m, int y, int k) throws IOException
@@ -1241,7 +1270,7 @@ public final class PDPageContentStream implements Closeable
         writeOperand(m);
         writeOperand(y);
         writeOperand(k);
-        writeOperator("K");
+        writeOperator(OperatorName.STROKING_COLOR_CMYK);
         setStrokingColorSpaceStack(PDDeviceCMYK.INSTANCE);
     }
 
@@ -1251,7 +1280,7 @@ public final class PDPageContentStream implements Closeable
      * @param g The gray value.
      * @throws IOException If an IO error occurs while writing to the stream.
      * @throws IllegalArgumentException If the parameter is invalid.
-     * @deprecated Use {@link #setStrokingColor(float)} instead.
+     * @deprecated Use {@link #setStrokingColor(float) setStrokingColor(g/255f)} instead.
      */
     @Deprecated
     public void setStrokingColor(int g) throws IOException
@@ -1291,7 +1320,7 @@ public final class PDPageContentStream implements Closeable
             throw new IllegalArgumentException("Parameter must be within 0..1, but is " + g);
         }
         writeOperand(g);
-        writeOperator("G");
+        writeOperator(OperatorName.STROKING_COLOR_GRAY);
         setStrokingColorSpaceStack(PDDeviceGray.INSTANCE);
     }
 
@@ -1307,7 +1336,7 @@ public final class PDPageContentStream implements Closeable
             nonStrokingColorSpaceStack.peek() != color.getColorSpace())
         {
             writeOperand(getName(color.getColorSpace()));
-            writeOperator("cs");
+            writeOperator(OperatorName.NON_STROKING_COLORSPACE);
             setNonStrokingColorSpaceStack(color.getColorSpace());
         }
 
@@ -1326,11 +1355,11 @@ public final class PDPageContentStream implements Closeable
             color.getColorSpace() instanceof PDDeviceN ||
             color.getColorSpace() instanceof PDICCBased)
         {
-            writeOperator("scn");
+            writeOperator(OperatorName.NON_STROKING_COLOR_N);
         }
         else
         {
-            writeOperator("sc");
+            writeOperator(OperatorName.NON_STROKING_COLOR);
         }
     }
 
@@ -1374,11 +1403,11 @@ public final class PDPageContentStream implements Closeable
             currentNonStrokingColorSpace instanceof PDPattern ||
             currentNonStrokingColorSpace instanceof PDICCBased)
         {
-            writeOperator("scn");
+            writeOperator(OperatorName.NON_STROKING_COLOR_N);
         }
         else
         {
-            writeOperator("sc");
+            writeOperator(OperatorName.NON_STROKING_COLOR);
         }
     }
 
@@ -1391,6 +1420,32 @@ public final class PDPageContentStream implements Closeable
      * @throws IOException If an IO error occurs while writing to the stream.
      * @throws IllegalArgumentException If the parameters are invalid.
      */
+    public void setNonStrokingColor(float r, float g, float b) throws IOException
+    {
+        if (isOutsideOneInterval(r) || isOutsideOneInterval(g) || isOutsideOneInterval(b))
+        {
+            throw new IllegalArgumentException("Parameters must be within 0..1, but are "
+                    + String.format("(%.2f,%.2f,%.2f)", r, g, b));
+        }
+        writeOperand(r);
+        writeOperand(g);
+        writeOperand(b);
+        writeOperator(OperatorName.NON_STROKING_RGB);
+        setNonStrokingColorSpaceStack(PDDeviceRGB.INSTANCE);
+    }
+
+    /**
+     * Set the non stroking color in the DeviceRGB color space. Range is 0..255.
+     *
+     * @param r The red value
+     * @param g The green value.
+     * @param b The blue value.
+     * @throws IOException If an IO error occurs while writing to the stream.
+     * @throws IllegalArgumentException If the parameters are invalid.
+     * @deprecated use
+     * {@link #setNonStrokingColor(float, float, float) setNonStrokingColor(r/255f, g/255f, b/255f)}
+     */
+    @Deprecated
     public void setNonStrokingColor(int r, int g, int b) throws IOException
     {
         if (isOutside255Interval(r) || isOutside255Interval(g) || isOutside255Interval(b))
@@ -1398,11 +1453,7 @@ public final class PDPageContentStream implements Closeable
             throw new IllegalArgumentException("Parameters must be within 0..255, but are "
                     + String.format("(%d,%d,%d)", r, g, b));
         }
-        writeOperand(r / 255f);
-        writeOperand(g / 255f);
-        writeOperand(b / 255f);
-        writeOperator("rg");
-        setNonStrokingColorSpaceStack(PDDeviceRGB.INSTANCE);
+        setNonStrokingColor(r / 255f, g / 255f, b / 255f);
     }
 
     /**
@@ -1462,7 +1513,7 @@ public final class PDPageContentStream implements Closeable
         writeOperand(m);
         writeOperand(y);
         writeOperand(k);
-        writeOperator("k");
+        writeOperator(OperatorName.NON_STROKING_CMYK);
         setNonStrokingColorSpaceStack(PDDeviceCMYK.INSTANCE);
     }
 
@@ -1472,6 +1523,7 @@ public final class PDPageContentStream implements Closeable
      * @param g The gray value.
      * @throws IOException If an IO error occurs while writing to the stream.
      * @throws IllegalArgumentException If the parameter is invalid.
+     * @deprecated use {@link #setNonStrokingColor(float) setNonStrokingColor(g/255f)}
      */
     public void setNonStrokingColor(int g) throws IOException
     {
@@ -1510,7 +1562,7 @@ public final class PDPageContentStream implements Closeable
             throw new IllegalArgumentException("Parameter must be within 0..1, but is " + g);
         }
         writeOperand(g);
-        writeOperator("g");
+        writeOperator(OperatorName.NON_STROKING_GRAY);
         setNonStrokingColorSpaceStack(PDDeviceGray.INSTANCE);
     }
 
@@ -1534,7 +1586,7 @@ public final class PDPageContentStream implements Closeable
         writeOperand(y);
         writeOperand(width);
         writeOperand(height);
-        writeOperator("re");
+        writeOperator(OperatorName.APPEND_RECT);
     }
 
     /**
@@ -1602,7 +1654,7 @@ public final class PDPageContentStream implements Closeable
         writeOperand(y2);
         writeOperand(x3);
         writeOperand(y3);
-        writeOperator("c");
+        writeOperator(OperatorName.CURVE_TO);
     }
 
     /**
@@ -1643,7 +1695,7 @@ public final class PDPageContentStream implements Closeable
         writeOperand(y2);
         writeOperand(x3);
         writeOperand(y3);
-        writeOperator("v");
+        writeOperator(OperatorName.CURVE_TO_REPLICATE_INITIAL_POINT);
     }
 
     /**
@@ -1684,7 +1736,7 @@ public final class PDPageContentStream implements Closeable
         writeOperand(y1);
         writeOperand(x3);
         writeOperand(y3);
-        writeOperator("y");
+        writeOperator(OperatorName.CURVE_TO_REPLICATE_FINAL_POINT);
     }
 
     /**
@@ -1703,7 +1755,7 @@ public final class PDPageContentStream implements Closeable
         }
         writeOperand(x);
         writeOperand(y);
-        writeOperator("m");
+        writeOperator(OperatorName.MOVE_TO);
     }
 
     /**
@@ -1722,7 +1774,7 @@ public final class PDPageContentStream implements Closeable
         }
         writeOperand(x);
         writeOperand(y);
-        writeOperator("l");
+        writeOperator(OperatorName.LINE_TO);
     }
 
     /**
@@ -1856,7 +1908,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: stroke is not allowed within a text block.");
         }
-        writeOperator("S");
+        writeOperator(OperatorName.STROKE_PATH);
     }
 
     /**
@@ -1871,7 +1923,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: closeAndStroke is not allowed within a text block.");
         }
-        writeOperator("s");
+        writeOperator(OperatorName.CLOSE_AND_STROKE);
     }
 
     /**
@@ -1911,7 +1963,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: fill is not allowed within a text block.");
         }
-        writeOperator("f");
+        writeOperator(OperatorName.FILL_NON_ZERO);
     }
 
     /**
@@ -1926,7 +1978,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: fillEvenOdd is not allowed within a text block.");
         }
-        writeOperator("f*");
+        writeOperator(OperatorName.FILL_EVEN_ODD);
     }
 
     /**
@@ -1943,7 +1995,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: fillAndStroke is not allowed within a text block.");
         }
-        writeOperator("B");
+        writeOperator(OperatorName.FILL_NON_ZERO_AND_STROKE);
     }
 
     /**
@@ -1960,7 +2012,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: fillAndStrokeEvenOdd is not allowed within a text block.");
         }
-        writeOperator("B*");
+        writeOperator(OperatorName.FILL_EVEN_ODD_AND_STROKE);
     }
 
     /**
@@ -1977,7 +2029,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: closeAndFillAndStroke is not allowed within a text block.");
         }
-        writeOperator("b");
+        writeOperator(OperatorName.CLOSE_FILL_NON_ZERO_AND_STROKE);
     }
 
     /**
@@ -1994,7 +2046,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: closeAndFillAndStrokeEvenOdd is not allowed within a text block.");
         }
-        writeOperator("b*");
+        writeOperator(OperatorName.CLOSE_FILL_EVEN_ODD_AND_STROKE);
     }
 
     /**
@@ -2012,7 +2064,7 @@ public final class PDPageContentStream implements Closeable
         }
 
         writeOperand(resources.add(shading));
-        writeOperator("sh");
+        writeOperator(OperatorName.SHADING_FILL);
     }
 
     /**
@@ -2039,7 +2091,7 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: closePath is not allowed within a text block.");
         }
-        writeOperator("h");
+        writeOperator(OperatorName.CLOSE_PATH);
     }
 
     /**
@@ -2059,17 +2111,17 @@ public final class PDPageContentStream implements Closeable
         }
         if (windingRule == PathIterator.WIND_NON_ZERO)
         {
-            writeOperator("W");
+            writeOperator(OperatorName.CLIP_NON_ZERO);
         }
         else if (windingRule == PathIterator.WIND_EVEN_ODD)
         {
-            writeOperator("W*");
+            writeOperator(OperatorName.CLIP_EVEN_ODD);
         }
         else
         {
             throw new IllegalArgumentException("Error: unknown value for winding rule");
         }
-        writeOperator("n");
+        writeOperator(OperatorName.ENDPATH);
     }
 
     /**
@@ -2084,10 +2136,10 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: clip is not allowed within a text block.");
         }
-        writeOperator("W");
+        writeOperator(OperatorName.CLIP_NON_ZERO);
         
         // end path without filling or stroking
-        writeOperator("n");
+        writeOperator(OperatorName.ENDPATH);
     }
 
     /**
@@ -2102,10 +2154,10 @@ public final class PDPageContentStream implements Closeable
         {
             throw new IllegalStateException("Error: clipEvenOdd is not allowed within a text block.");
         }
-        writeOperator("W*");
+        writeOperator(OperatorName.CLIP_EVEN_ODD);
         
         // end path without filling or stroking
-        writeOperator("n");
+        writeOperator(OperatorName.ENDPATH);
     }
 
     /**
@@ -2122,7 +2174,7 @@ public final class PDPageContentStream implements Closeable
             throw new IllegalStateException("Error: setLineWidth is not allowed within a text block.");
         }
         writeOperand(lineWidth);
-        writeOperator("w");
+        writeOperator(OperatorName.SET_LINE_WIDTH);
     }
 
     /**
@@ -2142,7 +2194,7 @@ public final class PDPageContentStream implements Closeable
         if (lineJoinStyle >= 0 && lineJoinStyle <= 2)
         {
             writeOperand(lineJoinStyle);
-            writeOperator("j");
+            writeOperator(OperatorName.SET_LINE_JOINSTYLE);
         }
         else
         {
@@ -2167,7 +2219,7 @@ public final class PDPageContentStream implements Closeable
         if (lineCapStyle >= 0 && lineCapStyle <= 2)
         {
             writeOperand(lineCapStyle);
-            writeOperator("J");
+            writeOperator(OperatorName.SET_LINE_CAPSTYLE);
         }
         else
         {
@@ -2196,7 +2248,7 @@ public final class PDPageContentStream implements Closeable
         }
         write("] ");
         writeOperand(phase);
-        writeOperator("d");
+        writeOperator(OperatorName.SET_LINE_DASHPATTERN);
     }
 
     /**
@@ -2216,7 +2268,7 @@ public final class PDPageContentStream implements Closeable
             throw new IllegalArgumentException("A miter limit <= 0 is invalid and will not render in Acrobat Reader");
         }
         writeOperand(miterLimit);
-        writeOperator("M");
+        writeOperator(OperatorName.SET_LINE_MITERLIMIT);
     }
 
     /**
@@ -2241,7 +2293,7 @@ public final class PDPageContentStream implements Closeable
     public void beginMarkedContent(COSName tag) throws IOException
     {
         writeOperand(tag);
-        writeOperator("BMC");
+        writeOperator(OperatorName.BEGIN_MARKED_CONTENT);
     }
 
     /**
@@ -2258,7 +2310,7 @@ public final class PDPageContentStream implements Closeable
     {
         writeOperand(tag);
         writeOperand(propsName);
-        writeOperator("BDC");
+        writeOperator(OperatorName.BEGIN_MARKED_CONTENT_SEQ);
     }
 
     /**
@@ -2273,7 +2325,7 @@ public final class PDPageContentStream implements Closeable
     {
         writeOperand(tag);
         writeOperand(resources.add(propertyList));
-        writeOperator("BDC");
+        writeOperator(OperatorName.BEGIN_MARKED_CONTENT_SEQ);
     }
 
     /**
@@ -2295,7 +2347,7 @@ public final class PDPageContentStream implements Closeable
      */
     public void endMarkedContent() throws IOException
     {
-        writeOperator("EMC");
+        writeOperator(OperatorName.END_MARKED_CONTENT);
     }
 
     /**
@@ -2385,7 +2437,7 @@ public final class PDPageContentStream implements Closeable
     public void setGraphicsStateParameters(PDExtendedGraphicsState state) throws IOException
     {
         writeOperand(resources.add(state));
-        writeOperator("gs");
+        writeOperator(OperatorName.SET_GRAPHICS_STATE_PARAMS);
     }
 
     /**
@@ -2558,7 +2610,7 @@ public final class PDPageContentStream implements Closeable
     public void setRenderingMode(RenderingMode rm) throws IOException
     {
         writeOperand(rm.intValue());
-        writeOperator("Tr");
+        writeOperator(OperatorName.SET_TEXT_RENDERINGMODE);
     }
 
     /**
@@ -2571,7 +2623,7 @@ public final class PDPageContentStream implements Closeable
     public void setCharacterSpacing(float spacing) throws IOException
     {
         writeOperand(spacing);
-        writeOperator("Tc");
+        writeOperator(OperatorName.SET_CHAR_SPACING);
     }
 
     /**
@@ -2590,7 +2642,7 @@ public final class PDPageContentStream implements Closeable
     public void setWordSpacing(float spacing) throws IOException
     {
         writeOperand(spacing);
-        writeOperator("Tw");
+        writeOperator(OperatorName.SET_WORD_SPACING);
     }
 
     /**
@@ -2603,7 +2655,7 @@ public final class PDPageContentStream implements Closeable
     public void setHorizontalScaling(float scale) throws IOException
     {
         writeOperand(scale);
-        writeOperator("Tz");
+        writeOperator(OperatorName.SET_TEXT_HORIZONTAL_SCALING);
     }
 
     /**
@@ -2617,6 +2669,6 @@ public final class PDPageContentStream implements Closeable
     public void setTextRise(float rise) throws IOException
     {
         writeOperand(rise);
-        writeOperator("Ts");
+        writeOperator(OperatorName.SET_TEXT_RISE);
     }
 }
