@@ -25,7 +25,6 @@ import java.security.AlgorithmParameters;
 import java.security.GeneralSecurityException;
 import java.security.InvalidKeyException;
 import java.security.KeyStoreException;
-import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.PrivateKey;
 import java.security.SecureRandom;
@@ -42,6 +41,7 @@ import javax.crypto.NoSuchPaddingException;
 import javax.crypto.SecretKey;
 
 import org.apache.pdfbox.cos.COSArray;
+import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSString;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.bouncycastle.asn1.ASN1InputStream;
@@ -80,7 +80,8 @@ public final class PublicKeySecurityHandler extends SecurityHandler
     /** The filter name. */
     public static final String FILTER = "Adobe.PubSec";
 
-    private static final String SUBFILTER = "adbe.pkcs7.s4";
+    private static final String SUBFILTER4 = "adbe.pkcs7.s4";
+    private static final String SUBFILTER5 = "adbe.pkcs7.s5";
 
     private PublicKeyProtectionPolicy policy = null;
     
@@ -151,14 +152,20 @@ public final class PublicKeySecurityHandler extends SecurityHandler
             byte[] envelopedData = null;
 
             // the bytes of each recipient in the recipients array
-            byte[][] recipientFieldsBytes = new byte[encryption.getRecipientsLength()][];
+            COSArray array = (COSArray) encryption.getCOSObject().getItem(COSName.RECIPIENTS);
+            if (array == null)
+            {
+                PDCryptFilterDictionary defaultCryptFilterDictionary = encryption.getDefaultCryptFilterDictionary();
+                array = (COSArray) defaultCryptFilterDictionary.getCOSObject().getItem(COSName.RECIPIENTS);
+            }
+            byte[][] recipientFieldsBytes = new byte[array.size()][];
+            //TODO encryption.getRecipientsLength() and getRecipientStringAt() should be deprecated
 
             int recipientFieldsLength = 0;
-            int i = 0;
             StringBuilder extraInfo = new StringBuilder();
-            for (; i < encryption.getRecipientsLength(); i++)
+            for (int i = 0; i < array.size(); i++)
             {
-                COSString recipientFieldString = encryption.getRecipientStringAt(i);
+                COSString recipientFieldString = (COSString) array.getObject(i);
                 byte[] recipientBytes = recipientFieldString.getBytes();
                 CMSEnvelopedData data = new CMSEnvelopedData(recipientBytes);
                 Collection<RecipientInformation> recipCertificatesIt = data.getRecipientInfos()
@@ -173,6 +180,8 @@ public final class PublicKeySecurityHandler extends SecurityHandler
                     {
                         foundRecipient = true;
                         PrivateKey privateKey = (PrivateKey) material.getPrivateKey();
+                        // might need to call setContentProvider() if we use PKI token, see
+                        // http://bouncy-castle.1462172.n4.nabble.com/CMSException-exception-unwrapping-key-key-invalid-unknown-key-type-passed-to-RSA-td4658109.html
                         envelopedData = ri.getContent(new JceKeyTransEnvelopedRecipient(privateKey));
                         break;
                     }
@@ -193,7 +202,7 @@ public final class PublicKeySecurityHandler extends SecurityHandler
             }
             if (!foundRecipient || envelopedData == null)
             {
-                throw new IOException("The certificate matches none of " + i
+                throw new IOException("The certificate matches none of " + array.size()
                         + " recipient entries" + extraInfo.toString());
             }
             if (envelopedData.length != 24)
@@ -226,8 +235,26 @@ public final class PublicKeySecurityHandler extends SecurityHandler
                 sha1InputOffset += recipientFieldsByte.length;
             }
 
-            MessageDigest md = MessageDigests.getSHA1();
-            byte[] mdResult = md.digest(sha1Input);
+            byte[] mdResult;
+            if (encryption.getVersion() == 4 || encryption.getVersion() == 5)
+            {
+                mdResult = MessageDigests.getSHA256().digest(sha1Input);
+
+                // detect whether AES encryption is used. This assumes that the encryption algo is 
+                // stored in the PDCryptFilterDictionary
+                // However, crypt filters are used only when V is 4 or 5.
+                PDCryptFilterDictionary defaultCryptFilterDictionary = encryption.getDefaultCryptFilterDictionary();
+                if (defaultCryptFilterDictionary != null)
+                {
+                    COSName cryptFilterMethod = defaultCryptFilterDictionary.getCryptFilterMethod();
+                    setAES(COSName.AESV2.equals(cryptFilterMethod) ||
+                           COSName.AESV3.equals(cryptFilterMethod));
+                }
+            }
+            else
+            {
+                mdResult = MessageDigests.getSHA1().digest(sha1Input);
+            }
 
             // we have the encryption key ...
             encryptionKey = new byte[this.keyLength / 8];
@@ -281,10 +308,6 @@ public final class PublicKeySecurityHandler extends SecurityHandler
     @Override
     public void prepareDocumentForEncryption(PDDocument doc) throws IOException
     {
-        if (keyLength == 256)
-        {
-            throw new IOException("256 bit key length is not supported yet for public key security");
-        }
         try
         {
             PDEncryption dictionary = doc.getEncryption();
@@ -295,15 +318,13 @@ public final class PublicKeySecurityHandler extends SecurityHandler
 
             dictionary.setFilter(FILTER);
             dictionary.setLength(this.keyLength);
-            dictionary.setVersion(2);
-            
+            int version = computeVersionNumber();
+            dictionary.setVersion(version);
+
             // remove CF, StmF, and StrF entries that may be left from a previous encryption
             dictionary.removeV45filters();
-            
-            dictionary.setSubFilter(SUBFILTER);
 
             // create the 20 bytes seed
-
             byte[] seed = new byte[20];
 
             KeyGenerator key;
@@ -319,42 +340,51 @@ public final class PublicKeySecurityHandler extends SecurityHandler
 
             key.init(192, new SecureRandom());
             SecretKey sk = key.generateKey();
-            System.arraycopy(sk.getEncoded(), 0, seed, 0, 20); // create the 20 bytes seed
-            
-            byte[][] recipientsField = computeRecipientsField(seed);
-            dictionary.setRecipients(recipientsField);
 
-            int sha1InputLength = seed.length;
+            // create the 20 bytes seed
+            System.arraycopy(sk.getEncoded(), 0, seed, 0, 20);
 
-            for(int j=0; j<dictionary.getRecipientsLength(); j++)
+            byte[][] recipientsFields = computeRecipientsField(seed);
+
+            int shaInputLength = seed.length;
+
+            for (byte[] field : recipientsFields)
             {
-                COSString string = dictionary.getRecipientStringAt(j);
-                sha1InputLength += string.getBytes().length;
+                shaInputLength += field.length;
             }
 
-            byte[] sha1Input = new byte[sha1InputLength];
+            byte[] shaInput = new byte[shaInputLength];
 
-            System.arraycopy(seed, 0, sha1Input, 0, 20);
+            System.arraycopy(seed, 0, shaInput, 0, 20);
 
-            int sha1InputOffset = 20;
+            int shaInputOffset = 20;
 
-            for(int j=0; j<dictionary.getRecipientsLength(); j++)
+            for (byte[] recipientsField : recipientsFields)
             {
-                COSString string = dictionary.getRecipientStringAt(j);
-                System.arraycopy(
-                    string.getBytes(), 0,
-                    sha1Input, sha1InputOffset, string.getBytes().length);
-                sha1InputOffset += string.getBytes().length;
+                System.arraycopy(recipientsField, 0, shaInput, shaInputOffset, recipientsField.length);
+                shaInputOffset += recipientsField.length;
             }
 
-            MessageDigest sha1 = MessageDigests.getSHA1();
-            byte[] mdResult = sha1.digest(sha1Input);
+            byte[] mdResult;
+            if (version == 4 || version == 5)
+            {
+                dictionary.setSubFilter(SUBFILTER5);
+                mdResult = MessageDigests.getSHA256().digest(shaInput);
+                COSName aesVName = version == 5 ? COSName.AESV3 : COSName.AESV2;
+                prepareEncryptionDictAES(dictionary, aesVName, recipientsFields);
+            }
+            else
+            {
+                dictionary.setSubFilter(SUBFILTER4);
+                mdResult = MessageDigests.getSHA1().digest(shaInput);
+                dictionary.setRecipients(recipientsFields);
+            }
 
             this.encryptionKey = new byte[this.keyLength/8];
             System.arraycopy(mdResult, 0, this.encryptionKey, 0, this.keyLength/8);
 
             doc.setEncryptionDictionary(dictionary);
-            doc.getDocument().setEncryptionDictionary(dictionary.getCOSDictionary());
+            doc.getDocument().setEncryptionDictionary(dictionary.getCOSObject());
         }
         catch(GeneralSecurityException e)
         {
@@ -362,6 +392,49 @@ public final class PublicKeySecurityHandler extends SecurityHandler
         }
     }
 
+    /**
+     * Computes the version number of the StandardSecurityHandler based on the encryption key
+     * length. See PDF Spec 1.6 p 93 and
+     * <a href="https://www.adobe.com/content/dam/acom/en/devnet/pdf/adobe_supplement_iso32000.pdf">PDF
+     * 1.7 Supplement ExtensionLevel: 3</a>
+     *
+     * @return The computed version number.
+     */
+    private int computeVersionNumber()
+    {
+        switch (keyLength)
+        {
+            case 40:
+                return 1;
+            case 128:
+                return 2; // prefer RC4 (AES 128 doesn't work yet)
+                //return 4; // prefer AES
+            case 256:
+                return 5;
+            default:
+                throw new IllegalArgumentException("key length must be 40, 128 or 256");
+        }
+    }
+
+    private void prepareEncryptionDictAES(PDEncryption encryptionDictionary, COSName aesVName, byte[][] recipients)
+    {
+        PDCryptFilterDictionary cryptFilterDictionary = new PDCryptFilterDictionary();
+        cryptFilterDictionary.setCryptFilterMethod(aesVName);
+        cryptFilterDictionary.setLength(keyLength);
+        COSArray array = new COSArray();
+        for (byte[] recipient : recipients)
+        {
+            array.add(new COSString(recipient));
+        }
+        cryptFilterDictionary.getCOSObject().setItem(COSName.RECIPIENTS, array);
+        array.setDirect(true);
+        encryptionDictionary.setDefaultCryptFilterDictionary(cryptFilterDictionary);
+        encryptionDictionary.setStreamFilterName(COSName.DEFAULT_CRYPT_FILTER);
+        encryptionDictionary.setStringFilterName(COSName.DEFAULT_CRYPT_FILTER);
+        cryptFilterDictionary.getCOSObject().setDirect(true);
+        setAES(true);
+    }
+
     private byte[][] computeRecipientsField(byte[] seed) throws GeneralSecurityException, IOException
     {
         byte[][] recipientsField = new byte[policy.getNumberOfRecipients()][];
@@ -405,14 +478,13 @@ public final class PublicKeySecurityHandler extends SecurityHandler
     private ASN1Primitive createDERForRecipient(byte[] in, X509Certificate cert)
             throws IOException, GeneralSecurityException
     {
-        String algorithm = "1.2.840.113549.3.2";
+        String algorithm = PKCSObjectIdentifiers.RC2_CBC.getId();
         AlgorithmParameterGenerator apg;
         KeyGenerator keygen;
         Cipher cipher;
         try
         {
-            apg = AlgorithmParameterGenerator.getInstance(algorithm,
-                    SecurityProvider.getProvider());
+            apg = AlgorithmParameterGenerator.getInstance(algorithm, SecurityProvider.getProvider());
             keygen = KeyGenerator.getInstance(algorithm, SecurityProvider.getProvider());
             cipher = Cipher.getInstance(algorithm, SecurityProvider.getProvider());
         }
