@@ -167,12 +167,18 @@ public final class StandardSecurityHandler extends SecurityHandler
     @Override
     public void prepareForDecryption(PDEncryption encryption, COSArray documentIDArray,
                                      DecryptionMaterial decryptionMaterial)
-                                     throws InvalidPasswordException, IOException
+                                     throws IOException
     {
         if(!(decryptionMaterial instanceof StandardDecryptionMaterial))
         {
             throw new IOException("Decryption material is not compatible with the document");
         }
+        
+        // This is only used with security version 4 and 5.
+        if (encryption.getVersion() >= 4) {
+	        setStreamFilterName(encryption.getStreamFilterName());
+	        setStringFilterName(encryption.getStreamFilterName());
+        }
         setDecryptMetadata(encryption.isEncryptMetaData());
         StandardDecryptionMaterial material = (StandardDecryptionMaterial)decryptionMaterial;
 
@@ -202,7 +208,12 @@ public final class StandardSecurityHandler extends SecurityHandler
             ue = encryption.getUserEncryptionKey();
             oe = encryption.getOwnerEncryptionKey();
         }
-        
+
+        if (dicRevision == 6)
+        {
+            password = SaslPrep.saslPrepQuery(password); // PDFBOX-4155
+        }
+
         AccessPermission currentAccessPermission;
 
         if( isOwnerPassword(password.getBytes(passwordCharset), userKey, ownerKey,
@@ -294,7 +305,7 @@ public final class StandardSecurityHandler extends SecurityHandler
         return documentIDBytes;
     }
 
-    // Algorithm 13: validate permissions ("Perms" field). Relaxed to accomodate buggy encoders
+    // Algorithm 13: validate permissions ("Perms" field). Relaxed to accommodate buggy encoders
     // https://www.adobe.com/content/dam/Adobe/en/devnet/acrobat/pdfs/adobe_supplement_iso32000.pdf
     private void validatePerms(PDEncryption encryption, int dicPermissions, boolean encryptMetadata) throws IOException
     {
@@ -338,7 +349,7 @@ public final class StandardSecurityHandler extends SecurityHandler
     /**
      * Prepare document for encryption.
      *
-     * @param document The documenet to encrypt.
+     * @param document The document to encrypt.
      *
      * @throws IOException If there is an error accessing data.
      */
@@ -372,7 +383,7 @@ public final class StandardSecurityHandler extends SecurityHandler
         {
             userPassword = "";
         }
- 
+
         // If no owner password is set, use the user password instead.
         if (ownerPassword.isEmpty())
         {
@@ -387,6 +398,9 @@ public final class StandardSecurityHandler extends SecurityHandler
 
         if (revision == 6)
         {
+            // PDFBOX-4155
+            ownerPassword = SaslPrep.saslPrepStored(ownerPassword);
+            userPassword = SaslPrep.saslPrepStored(userPassword);
             prepareEncryptionDictRev6(ownerPassword, userPassword, encryptionDictionary, permissionInt);
         }
         else
@@ -396,7 +410,7 @@ public final class StandardSecurityHandler extends SecurityHandler
         }
 
         document.setEncryptionDictionary( encryptionDictionary );
-        document.getDocument().setEncryptionDictionary(encryptionDictionary.getCOSDictionary());
+        document.getDocument().setEncryptionDictionary(encryptionDictionary.getCOSObject());
     }
 
     private void prepareEncryptionDictRev6(String ownerPassword, String userPassword,
