@@ -127,7 +127,7 @@ public class FDFDocument implements Closeable
     {
         FDFCatalog retval = null;
         COSDictionary trailer = document.getTrailer();
-        COSDictionary root = (COSDictionary) trailer.getDictionaryObject(COSName.ROOT);
+        COSDictionary root = trailer.getCOSDictionary(COSName.ROOT);
         if (root == null)
         {
             retval = new FDFCatalog();
@@ -238,8 +238,7 @@ public class FDFDocument implements Closeable
      */
     public static FDFDocument loadXFDF(InputStream input) throws IOException
     {
-        Document doc = XMLUtil.parse(input);
-        return new FDFDocument(doc);
+        return new FDFDocument(org.apache.pdfbox.util.XMLUtil.parse(input));
     }
 
     /**
