@@ -18,7 +18,6 @@ package org.apache.pdfbox.pdmodel.font;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.URL;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -108,13 +107,12 @@ final class FontMapperImpl implements FontMapper
         
         try
         {
-            String ttfName = "org/apache/pdfbox/resources/ttf/LiberationSans-Regular.ttf";
-            URL url = FontMapper.class.getClassLoader().getResource(ttfName);
-            if (url == null)
+            String ttfName = "/org/apache/pdfbox/resources/ttf/LiberationSans-Regular.ttf";
+            InputStream ttfStream = FontMapper.class.getResourceAsStream(ttfName);
+            if (ttfStream == null)
             {
                 throw new IOException("Error loading resource: " + ttfName);
             }
-            InputStream ttfStream = url.openStream();
             TTFParser ttfParser = new TTFParser();
             lastResortFont = ttfParser.parse(ttfStream);
         }
@@ -184,7 +182,7 @@ final class FontMapperImpl implements FontMapper
         names.add(postScriptName);
      
         // remove hyphens (e.g. Arial-Black -> ArialBlack)
-        names.add(postScriptName.replaceAll("-", ""));
+        names.add(postScriptName.replace("-", ""));
      
         return names;
     }
@@ -217,7 +215,7 @@ final class FontMapperImpl implements FontMapper
      */
     private List<String> getSubstitutes(String postScriptName)
     {
-        List<String> subs = substitutes.get(postScriptName.replaceAll(" ", ""));
+        List<String> subs = substitutes.get(postScriptName.replace(" ", ""));
         if (subs != null)
         {
             return subs;
@@ -421,7 +419,7 @@ final class FontMapperImpl implements FontMapper
         }
 
         // remove hyphens (e.g. Arial-Black -> ArialBlack)
-        info = getFont(format, postScriptName.replaceAll("-", ""));
+        info = getFont(format, postScriptName.replace("-", ""));
         if (info != null)
         {
             return info.getFont();
@@ -438,7 +436,7 @@ final class FontMapperImpl implements FontMapper
         }
 
         // then try converting Windows names e.g. (ArialNarrow,Bold) -> (ArialNarrow-Bold)
-        info = getFont(format, postScriptName.replaceAll(",", "-"));
+        info = getFont(format, postScriptName.replace(",", "-"));
         if (info != null)
         {
             return info.getFont();
