@@ -21,7 +21,6 @@ import java.io.InputStream;
 import java.util.HashMap;
 import java.util.Map;
 
-import org.apache.fontbox.util.BoundingBox;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSDictionary;
@@ -30,7 +29,6 @@ import org.apache.pdfbox.cos.COSNumber;
 import org.apache.pdfbox.cos.COSStream;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.common.COSObjectable;
-import org.apache.pdfbox.util.Matrix;
 import org.apache.pdfbox.util.Vector;
 
 /**
@@ -61,7 +59,7 @@ public abstract class PDCIDFont implements COSObjectable, PDFontLike, PDVectorFo
      *
      * @param fontDictionary The font dictionary according to the PDF specification.
      */
-    PDCIDFont(COSDictionary fontDictionary, PDType0Font parent) throws IOException
+    PDCIDFont(COSDictionary fontDictionary, PDType0Font parent)
     {
         this.dict = fontDictionary;
         this.parent = parent;
@@ -200,9 +198,6 @@ public abstract class PDCIDFont implements COSObjectable, PDFontLike, PDVectorFo
         return fontDescriptor;
     }
 
-    @Override
-    public abstract Matrix getFontMatrix();
-
     /**
      * Returns the Type 0 font which is the parent of this font.
      *
@@ -213,9 +208,6 @@ public abstract class PDCIDFont implements COSObjectable, PDFontLike, PDVectorFo
         return parent;
     }
 
-    @Override
-    public abstract BoundingBox getBoundingBox() throws IOException;
-
     /**
      * This will get the default width. The default value for the default width is 1000.
      *
@@ -293,9 +285,6 @@ public abstract class PDCIDFont implements COSObjectable, PDFontLike, PDVectorFo
         return w1y;
     }
 
-    @Override
-    public abstract float getHeight(int code) throws IOException;
-
     @Override
     public float getWidth(int code) throws IOException
     {
@@ -305,12 +294,6 @@ public abstract class PDCIDFont implements COSObjectable, PDFontLike, PDVectorFo
         return getWidthForCID(codeToCID(code));
     }
 
-    @Override
-    public abstract float getWidthFromFont(int code) throws IOException;
-
-    @Override
-    public abstract boolean isEmbedded();
-
     @Override
     // todo: this method is highly suspicious, the average glyph width is not usually a good metric
     public float getAverageFontWidth()
