@@ -141,34 +141,32 @@ public class PDDeviceN extends PDSpecialColorSpace
 
         // spot colorants
         spotColorSpaces = new PDSeparation[numColorants];
-        if (attributes.getColorants() != null)
+
+        // spot color spaces
+        Map<String, PDSeparation> spotColorants = attributes.getColorants();
+
+        // map each colorant to the corresponding spot color space
+        for (int c = 0; c < numColorants; c++)
         {
-            // spot color spaces
-            Map<String, PDSeparation> spotColorants = attributes.getColorants();
-
-            // map each colorant to the corresponding spot color space
-            for (int c = 0; c < numColorants; c++)
+            String name = colorantNames.get(c);
+            PDSeparation spot = spotColorants.get(name);
+            if (spot != null)
             {
-                String name = colorantNames.get(c);
-                PDSeparation spot = spotColorants.get(name);
-                if (spot != null)
-                {
-                    // spot colorant
-                    spotColorSpaces[c] = spot;
+                // spot colorant
+                spotColorSpaces[c] = spot;
 
-                    // spot colors may replace process colors with same name
-                    // providing that the subtype is not NChannel.
-                    if (!isNChannel())
-                    {
-                        colorantToComponent[c] = -1;
-                    }
-                }
-                else
+                // spot colors may replace process colors with same name
+                // providing that the subtype is not NChannel.
+                if (!isNChannel())
                 {
-                    // process colorant
-                    spotColorSpaces[c] = null;
+                    colorantToComponent[c] = -1;
                 }
             }
+            else
+            {
+                // process colorant
+                spotColorSpaces[c] = null;
+            }
         }
     }
 
@@ -282,9 +280,9 @@ public class PDDeviceN extends PDSpecialColorSpace
     //
     private BufferedImage toRGBWithTintTransform(WritableRaster raster) throws IOException
     {
-        // map only in use if one color component
-        Map<Float,int[]> map1 = new HashMap<Float,int[]>();
-        float key = 0;
+        // cache color mappings
+        Map<String, int[]> map1 = new HashMap<String, int[]>();
+        String key = null;
 
         int width = raster.getWidth();
         int height = raster.getHeight();
@@ -301,27 +299,23 @@ public class PDDeviceN extends PDSpecialColorSpace
             for (int x = 0; x < width; x++)
             {
                 raster.getPixel(x, y, src);
-                if (numSrcComponents == 1)
+                // use a string representation as key
+                key = Float.toString(src[0]);
+                for (int s = 1; s < numSrcComponents; s++)
                 {
-                    int[] pxl = map1.get(src[0]);
-                    if (pxl != null)
-                    {
-                        rgbRaster.setPixel(x, y, pxl);
-                        continue;
-                    }
-                    else
-                    {
-                        // need to remember key because src is modified
-                        key = src[0];
-                    }
+                    key += "#" + Float.toString(src[s]);
+                }
+                int[] pxl = map1.get(key);
+                if (pxl != null)
+                {
+                    rgbRaster.setPixel(x, y, pxl);
+                    continue;
                 }
-
                 // scale to 0..1
                 for (int s = 0; s < numSrcComponents; s++)
                 {
                     src[s] = src[s] / 255;
                 }
-
                 // convert to alternate color space via tint transform
                 float[] result = tintTransform.eval(src);
                 
@@ -333,12 +327,8 @@ public class PDDeviceN extends PDSpecialColorSpace
                     // scale to 0..255
                     rgb[s] = (int) (rgbFloat[s] * 255f);
                 }                
-
-                if (numSrcComponents == 1)
-                {
-                    // must clone because rgb is reused
-                    map1.put(key, rgb.clone());
-                }
+                // must clone because rgb is reused
+                map1.put(key, rgb.clone());
 
                 rgbRaster.setPixel(x, y, rgb);
             }
