@@ -75,7 +75,26 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     static
     {
         String cmmProperty = System.getProperty("sun.java2d.cmm");
-        IS_KCMS = !isMinJdk8() || "sun.java2d.cmm.kcms.KcmsServiceProvider".equals(cmmProperty);
+        boolean result = false;
+        if (!isMinJdk8())
+        {
+            // always KCMS but class has different name
+            result = true;
+        }
+        else if ("sun.java2d.cmm.kcms.KcmsServiceProvider".equals(cmmProperty))
+        {
+            try
+            {
+                Class.forName("sun.java2d.cmm.kcms.KcmsServiceProvider");
+                result = true;
+            }
+            catch (ClassNotFoundException e)
+            {
+                // KCMS not available
+            }
+        }
+        // else maybe KCMS was available, but not wished
+        IS_KCMS = result;
     }
 
     /**
@@ -494,11 +513,11 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         switch (alternateColorSpace.getNumberOfComponents())
         {
             case 1:
-                return ICC_ColorSpace.TYPE_GRAY;
+                return ColorSpace.TYPE_GRAY;
             case 3:
-                return ICC_ColorSpace.TYPE_RGB;
+                return ColorSpace.TYPE_RGB;
             case 4:
-                return ICC_ColorSpace.TYPE_CMYK;
+                return ColorSpace.TYPE_CMYK;
             default:
                 // should not happen as all ICC color spaces in PDF must have 1,3, or 4 components
                 return -1;
