@@ -202,11 +202,17 @@ public final class PDImageXObject extends PDXObject implements PDImage
 
     /**
      * Create a PDImageXObject from an image file. The file format is determined by the file name
-     * suffix. The following suffixes are supported: jpg, jpeg, tif, tiff, gif, bmp and png. This is
+     * suffix. The following suffixes are supported: JPG, JPEG, TIF, TIFF, GIF, BMP and PNG. This is
      * a convenience method that calls {@link JPEGFactory#createFromStream},
      * {@link CCITTFactory#createFromFile} or {@link ImageIO#read} combined with
      * {@link LosslessFactory#createFromImage}. (The later can also be used to create a
-     * PDImageXObject from a BufferedImage).
+     * PDImageXObject from a BufferedImage). Starting with 2.0.18, this call will create an image
+     * directly from a PNG file without decoding it (when possible), which is faster. However the
+     * result size depends on the compression skill of the software that created the PNG file. If
+     * file size or bandwidth are important to you or to your clients, then create your PNG files
+     * with a tool that has implemented the
+     * <a href="https://blog.codinghorror.com/zopfli-optimization-literally-free-bandwidth/">Zopfli
+     * algorithm</a>, or use the two-step process mentioned above.
      *
      * @param file the image file.
      * @param doc the document that shall use this PDImageXObject.
@@ -226,10 +232,16 @@ public final class PDImageXObject extends PDXObject implements PDImage
         String ext = name.substring(dot + 1).toLowerCase();
         if ("jpg".equals(ext) || "jpeg".equals(ext))
         {
-            FileInputStream fis = new FileInputStream(file);
-            PDImageXObject imageXObject = JPEGFactory.createFromStream(doc, fis);
-            fis.close();
-            return imageXObject;
+            FileInputStream fis = null;
+            try
+            {
+                fis = new FileInputStream(file);
+                return JPEGFactory.createFromStream(doc, fis);
+            }
+            finally
+            {
+                IOUtils.closeQuietly(fis);
+            }
         }
         if ("tif".equals(ext) || "tiff".equals(ext))
         {
@@ -245,11 +257,17 @@ public final class PDImageXObject extends PDXObject implements PDImage
 
     /**
      * Create a PDImageXObject from an image file. The file format is determined by the file
-     * content. The following file types are supported: jpg, jpeg, tif, tiff, gif, bmp and png. This
+     * content. The following file types are supported: JPG, JPEG, TIF, TIFF, GIF, BMP and PNG. This
      * is a convenience method that calls {@link JPEGFactory#createFromStream},
      * {@link CCITTFactory#createFromFile} or {@link ImageIO#read} combined with
      * {@link LosslessFactory#createFromImage}. (The later can also be used to create a
-     * PDImageXObject from a BufferedImage).
+     * PDImageXObject from a BufferedImage). Starting with 2.0.18, this call will create an image
+     * directly from a png file without decoding it (when possible), which is faster. However the
+     * result size depends on the compression skill of the software that created the PNG file. If
+     * file size or bandwidth are important to you or to your clients, then create your PNG files
+     * with a tool that has implemented the
+     * <a href="https://blog.codinghorror.com/zopfli-optimization-literally-free-bandwidth/">Zopfli
+     * algorithm</a>, or use the two-step process mentioned above.
      *
      * @param file the image file.
      * @param doc the document that shall use this PDImageXObject.
@@ -315,11 +333,17 @@ public final class PDImageXObject extends PDXObject implements PDImage
 
     /**
      * Create a PDImageXObject from bytes of an image file. The file format is determined by the
-     * file content. The following file types are supported: jpg, jpeg, tif, tiff, gif, bmp and png.
+     * file content. The following file types are supported: JPG, JPEG, TIF, TIFF, GIF, BMP and PNG.
      * This is a convenience method that calls {@link JPEGFactory#createFromByteArray},
      * {@link CCITTFactory#createFromFile} or {@link ImageIO#read} combined with
      * {@link LosslessFactory#createFromImage}. (The later can also be used to create a
-     * PDImageXObject from a BufferedImage).
+     * PDImageXObject from a BufferedImage). Starting with 2.0.18, this call will create an image
+     * directly from a PNG file without decoding it (when possible), which is faster. However the
+     * result size depends on the compression skill of the software that created the PNG file. If
+     * file size or bandwidth are important to you or to your clients, then create your PNG files
+     * with a tool that has implemented the
+     * <a href="https://blog.codinghorror.com/zopfli-optimization-literally-free-bandwidth/">Zopfli
+     * algorithm</a>, or use the two-step process mentioned above.
      *
      * @param byteArray bytes from an image file.
      * @param document the document that shall use this PDImageXObject.
@@ -349,6 +373,15 @@ public final class PDImageXObject extends PDXObject implements PDImage
         {
             return JPEGFactory.createFromByteArray(document, byteArray);
         }
+        if (fileType.equals(FileType.PNG))
+        {
+            // Try to directly convert the image without recoding it.
+            PDImageXObject image = PNGConverter.convertPNGImage(document, byteArray);
+            if (image != null)
+            {
+                return image;
+            }
+        }
         if (fileType.equals(FileType.TIFF))
         {
             try
@@ -379,7 +412,7 @@ public final class PDImageXObject extends PDXObject implements PDImage
      */
     public PDMetadata getMetadata()
     {
-        COSStream cosStream = (COSStream) getCOSObject().getDictionaryObject(COSName.METADATA);
+        COSStream cosStream = getCOSObject().getCOSStream(COSName.METADATA);
         if (cosStream != null)
         {
             return new PDMetadata(cosStream);
@@ -398,11 +431,12 @@ public final class PDImageXObject extends PDXObject implements PDImage
 
     /**
      * Returns the key of this XObject in the structural parent tree.
-     * @return this object's key the structural parent tree
+     *
+     * @return this object's key the structural parent tree or -1 if there isn't any.
      */
     public int getStructParent()
     {
-        return getCOSObject().getInt(COSName.STRUCT_PARENT, 0);
+        return getCOSObject().getInt(COSName.STRUCT_PARENT);
     }
 
     /**
@@ -470,6 +504,13 @@ public final class PDImageXObject extends PDXObject implements PDImage
         return image;
     }
 
+    /**
+     * Extract the matte color from a softmask.
+     * 
+     * @param softMask
+     * @return the matte color.
+     * @throws IOException if the color conversion fails.
+     */
     private float[] extractMatte(PDImageXObject softMask) throws IOException
     {
         COSBase base = softMask.getCOSObject().getItem(COSName.MATTE);
@@ -514,7 +555,6 @@ public final class PDImageXObject extends PDXObject implements PDImage
     // soft mask: RGB + Gray -> ARGB
     private BufferedImage applyMask(BufferedImage image, BufferedImage mask,
                                     boolean isSoft, float[] matte)
-            throws IOException
     {
         if (mask == null)
         {
@@ -535,6 +575,11 @@ public final class PDImageXObject extends PDXObject implements PDImage
             height = mask.getHeight();
             image = scaleImage(image, width, height);
         }
+        else if (image.getRaster().getPixel(0, 0, (int[]) null).length < 3)
+        {
+            // PDFBOX-4470 bitonal image has only one element => copy into RGB
+            image = scaleImage(image, width, height);
+        }
 
         // compose to ARGB
         BufferedImage masked = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
@@ -554,7 +599,7 @@ public final class PDImageXObject extends PDXObject implements PDImage
                 rgba[0] = rgb[0];
                 rgba[1] = rgb[1];
                 rgba[2] = rgb[2];
-                
+
                 alphaPixel = alpha.getPixel(x, y, alphaPixel);
                 if (isSoft)
                 {
@@ -617,7 +662,7 @@ public final class PDImageXObject extends PDXObject implements PDImage
         }
         else
         {
-            COSStream cosStream = (COSStream) getCOSObject().getDictionaryObject(COSName.MASK);
+            COSStream cosStream = getCOSObject().getCOSStream(COSName.MASK);
             if (cosStream != null)
             {
                 // always DeviceGray
@@ -648,7 +693,7 @@ public final class PDImageXObject extends PDXObject implements PDImage
      */
     public PDImageXObject getSoftMask() throws IOException
     {
-        COSStream cosStream = (COSStream) getCOSObject().getDictionaryObject(COSName.SMASK);
+        COSStream cosStream = getCOSObject().getCOSStream(COSName.SMASK);
         if (cosStream != null)
         {
             // always DeviceGray
@@ -745,6 +790,8 @@ public final class PDImageXObject extends PDXObject implements PDImage
     public void setColorSpace(PDColorSpace cs)
     {
         getCOSObject().setItem(COSName.COLORSPACE, cs != null ? cs.getCOSObject() : null);
+        colorSpace = null;
+        cachedImage = null;
     }
 
     @Override
