@@ -178,7 +178,7 @@ final class SampledImageReader
      * @param region The region of the source image to get, or null if the entire image is needed.
      *               The actual region will be clipped to the dimensions of the source image.
      * @param subsampling The amount of rows and columns to advance for every output pixel, a value
-     *                  of 1 meaning every pixel will be read
+     * of 1 meaning every pixel will be read. It must not be larger than the image width or height.
      * @param colorKey an optional color key mask
      * @return content of this image as an (A)RGB buffered image
      * @throws IOException if the image cannot be read
@@ -205,26 +205,32 @@ final class SampledImageReader
             throw new IOException("image width and height must be positive");
         }
 
-        if (bitsPerComponent == 1 && colorKey == null && numComponents == 1)
+        try
         {
-            return from1Bit(pdImage, clipped, subsampling, width, height);
-        }
+            if (bitsPerComponent == 1 && colorKey == null && numComponents == 1)
+            {
+                return from1Bit(pdImage, clipped, subsampling, width, height);
+            }
 
-        //
-        // An AWT raster must use 8/16/32 bits per component. Images with < 8bpc
-        // will be unpacked into a byte-backed raster. Images with 16bpc will be reduced
-        // in depth to 8bpc as they will be drawn to TYPE_INT_RGB images anyway. All code
-        // in PDColorSpace#toRGBImage expects an 8-bit range, i.e. 0-255.
-        // Interleaved raster allows chunk-copying for 8-bit images.
-        WritableRaster raster = Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE, width, height,
-                numComponents, new Point(0, 0));
-        final float[] defaultDecode = pdImage.getColorSpace().getDefaultDecode(8);
-        if (bitsPerComponent == 8 && Arrays.equals(decode, defaultDecode) && colorKey == null)
+            // An AWT raster must use 8/16/32 bits per component. Images with < 8bpc
+            // will be unpacked into a byte-backed raster. Images with 16bpc will be reduced
+            // in depth to 8bpc as they will be drawn to TYPE_INT_RGB images anyway. All code
+            // in PDColorSpace#toRGBImage expects an 8-bit range, i.e. 0-255.
+            // Interleaved raster allows chunk-copying for 8-bit images.
+            WritableRaster raster = Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE, width, height,
+                    numComponents, new Point(0, 0));
+            final float[] defaultDecode = pdImage.getColorSpace().getDefaultDecode(8);
+            if (bitsPerComponent == 8 && Arrays.equals(decode, defaultDecode) && colorKey == null)
+            {
+                // convert image, faster path for non-decoded, non-colormasked 8-bit images
+                return from8bit(pdImage, raster, clipped, subsampling, width, height);
+            }
+            return fromAny(pdImage, raster, colorKey, clipped, subsampling, width, height);
+        }
+        catch (NegativeArraySizeException ex)
         {
-            // convert image, faster path for non-decoded, non-colormasked 8-bit images
-            return from8bit(pdImage, raster, clipped, subsampling, width, height);
+            throw new IOException(ex);
         }
-        return fromAny(pdImage, raster, colorKey, clipped, subsampling, width, height);
     }
 
     private static BufferedImage from1Bit(PDImage pdImage, Rectangle clipped, final int subsampling,
@@ -243,6 +249,9 @@ final class SampledImageReader
         InputStream iis = null;
         try
         {
+            // create stream
+            iis = pdImage.createInputStream(options);
+
             final int inputWidth;
             final int startx;
             final int starty;
@@ -282,9 +291,6 @@ final class SampledImageReader
             output = ((DataBufferByte) raster.getDataBuffer()).getData();
             final boolean isIndexed = colorSpace instanceof PDIndexed;
 
-            // create stream
-            iis = pdImage.createInputStream(options);
-
             int rowLen = inputWidth / 8;
             if (inputWidth % 8 > 0)
             {
@@ -464,6 +470,8 @@ final class SampledImageReader
         ImageInputStream iis = null;
         try
         {
+            iis = new MemoryCacheImageInputStream(pdImage.createInputStream(options));
+
             final int inputWidth;
             final int startx;
             final int starty;
@@ -490,7 +498,6 @@ final class SampledImageReader
             }
             // create stream
             final float sampleMax = (float) Math.pow(2, bitsPerComponent) - 1f;
-            iis = new MemoryCacheImageInputStream(pdImage.createInputStream(options));
             final boolean isIndexed = colorSpace instanceof PDIndexed;
 
             // init color key mask
@@ -593,7 +600,6 @@ final class SampledImageReader
 
     // color key mask: RGB + Binary -> ARGB
     private static BufferedImage applyColorKeyMask(BufferedImage image, BufferedImage mask)
-            throws IOException
     {
         int width = image.getWidth();
         int height = image.getHeight();
