@@ -27,6 +27,7 @@ import org.apache.pdfbox.cos.COSDictionary;
 import org.apache.pdfbox.cos.COSInteger;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSNumber;
+import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDPage;
 import org.apache.pdfbox.pdmodel.common.COSObjectable;
 import org.apache.pdfbox.pdmodel.common.PDRectangle;
@@ -85,6 +86,11 @@ public abstract class PDAnnotation implements COSObjectable
      * An annotation flag.
      */
     private static final int FLAG_TOGGLE_NO_VIEW = 1 << 8;
+    /**
+     * An annotation flag.
+     * @see #setLockedContents(boolean)
+     */
+    private static final int FLAG_LOCKED_CONTENTS = 1 << 9;
 
     private final COSDictionary dictionary;
 
@@ -511,6 +517,35 @@ public abstract class PDAnnotation implements COSObjectable
         getCOSObject().setFlag(COSName.F, FLAG_TOGGLE_NO_VIEW, toggleNoView);
     }
 
+    /**
+     * Get the LockedContents flag.
+     *
+     * @return The LockedContents flag.
+     * @see #setLockedContents(boolean)
+     */
+    public boolean isLockedContents()
+    {
+        return getCOSObject().getFlag(COSName.F, FLAG_LOCKED_CONTENTS);
+    }
+
+    /**
+     * Set the LockedContents flag. If set, do not allow the contents of the annotation to be
+     * modified by the user. This flag does not restrict deletion of the annotation or changes to
+     * other annotation properties, such as position and size.
+     *
+     * @param lockedContents The new LockedContents flag value.
+     * @see
+     * <a href="https://www.adobe.com/content/dam/acom/en/devnet/acrobat/pdfs/PDF32000_2008.pdf#page=393">PDF
+     * 32000-1:2008 12.5.3, Table 165</a>
+     * @see #isLockedContents()
+     * @see #FLAG_LOCKED_CONTENTS
+     * @since PDF 1.7
+     */
+    public void setLockedContents(boolean lockedContents)
+    {
+        getCOSObject().setFlag(COSName.F, FLAG_LOCKED_CONTENTS, lockedContents);
+    }
+
     /**
      * Get the "contents" of the field.
      * 
@@ -534,7 +569,7 @@ public abstract class PDAnnotation implements COSObjectable
     /**
      * This will retrieve the date and time the annotation was modified.
      * 
-     * @return the modified date/time (often in date format, but can be an arbitary string).
+     * @return the modified date/time (often in date format, but can be an arbitrary string).
      */
     public String getModifiedDate()
     {
@@ -774,6 +809,17 @@ public abstract class PDAnnotation implements COSObjectable
         return null;
     }
 
+    /**
+     * Create the appearance entry for this annotation. Not having it may prevent display in some
+     * viewers. This method is for overriding in subclasses, the default implementation does
+     * nothing.
+     * 
+     * @param document
+     */
+    public void constructAppearances(PDDocument document)
+    {
+    }
+
     /**
      * Create the appearance entry for this annotation. Not having it may prevent display in some
      * viewers. This method is for overriding in subclasses, the default implementation does
