@@ -145,8 +145,9 @@ public abstract class PDButton extends PDTerminalField
     }
 
     /**
-     * Sets the selected option given its name.
-     * 
+     * Sets the selected option given its name. It also tries to update the visual appearance,
+     * unless {@link PDAcroForm#getNeedAppearances()} is true.
+     *
      * @param value Name of option to select
      * @throws IOException if the value could not be set
      * @throws IllegalArgumentException if the value is not a valid option.
@@ -365,7 +366,7 @@ public abstract class PDButton extends PDTerminalField
      * @param value Name of radio button to select
      * @throws IllegalArgumentException if the value is not a valid option.
      */
-    void checkValue(String value) throws IllegalArgumentException
+    void checkValue(String value)
     {
         Set<String> onValues = getOnValues();
         if (COSName.Off.getName().compareTo(value) != 0 && !onValues.contains(value))
