@@ -20,6 +20,7 @@ import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
+import java.awt.GraphicsConfiguration;
 import java.awt.GraphicsDevice;
 import java.awt.Paint;
 import java.awt.Point;
@@ -43,13 +44,14 @@ import java.awt.image.DataBufferByte;
 import java.awt.image.Raster;
 import java.awt.image.WritableRaster;
 import java.io.IOException;
+import java.util.ArrayDeque;
 import java.util.ArrayList;
+import java.util.Deque;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.Stack;
 import java.util.StringTokenizer;
 
 import org.apache.commons.logging.Log;
@@ -140,6 +142,9 @@ public class PageDrawer extends PDFGraphicsStreamEngine
     // last clipping path
     private Area lastClip;
 
+    // clip when drawPage() is called, can be null, must be intersected when clipping
+    private Shape initialClip;
+
     // shapes of glyphs being drawn to be used for clipping
     private List<Shape> textClippings;
 
@@ -148,9 +153,9 @@ public class PageDrawer extends PDFGraphicsStreamEngine
     
     private final TilingPaintFactory tilingPaintFactory = new TilingPaintFactory(this);
     
-    private final Stack<TransparencyGroup> transparencyGroupStack = new Stack<TransparencyGroup>();
+    private final Deque<TransparencyGroup> transparencyGroupStack = new ArrayDeque<TransparencyGroup>();
     
-    // if greater zero the content is hidden and wil not be rendered
+    // if greater zero the content is hidden and will not be rendered
     private int nestedHiddenOCGCount;
 
     private final RenderDestination destination;
@@ -250,6 +255,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
     {
         graphics = (Graphics2D) g;
         xform = graphics.getTransform();
+        initialClip = graphics.getClip();
         this.pageSize = pageSize;
 
         setRenderingHints();
@@ -283,28 +289,31 @@ public class PageDrawer extends PDFGraphicsStreamEngine
     void drawTilingPattern(Graphics2D g, PDTilingPattern pattern, PDColorSpace colorSpace,
                                   PDColor color, Matrix patternMatrix) throws IOException
     {
-        Graphics2D oldGraphics = graphics;
+        Graphics2D savedGraphics = graphics;
         graphics = g;
 
-        GeneralPath oldLinePath = linePath;
+        GeneralPath savedLinePath = linePath;
         linePath = new GeneralPath();
-        int oldClipWindingRule = clipWindingRule;
+        int savedClipWindingRule = clipWindingRule;
         clipWindingRule = -1;
 
-        Area oldLastClip = lastClip;
+        Area savedLastClip = lastClip;
         lastClip = null;
+        Shape savedInitialClip = initialClip;
+        initialClip = null;
 
-        boolean oldFlipTG = flipTG;
+        boolean savedFlipTG = flipTG;
         flipTG = true;
 
         setRenderingHints();
         processTilingPattern(pattern, color, colorSpace, patternMatrix);
 
-        flipTG = oldFlipTG;
-        graphics = oldGraphics;
-        linePath = oldLinePath;
-        lastClip = oldLastClip;
-        clipWindingRule = oldClipWindingRule;
+        flipTG = savedFlipTG;
+        graphics = savedGraphics;
+        linePath = savedLinePath;
+        lastClip = savedLastClip;
+        initialClip = savedInitialClip;
+        clipWindingRule = savedClipWindingRule;
     }
 
     private float clampColor(float color)
@@ -370,6 +379,11 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         if (clippingPath != lastClip)
         {
             graphics.setClip(clippingPath);
+            if (initialClip != null)
+            {
+                // apply the remembered initial clip, but transform it first
+                //TODO see PDFBOX-4583
+            }
             lastClip = clippingPath;
         }
     }
@@ -658,7 +672,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
     }
 
     // returns the image adjusted for applySoftMaskToPaint().
-    private BufferedImage adjustImage(BufferedImage gray) throws IOException
+    private BufferedImage adjustImage(BufferedImage gray)
     {
         AffineTransform at = new AffineTransform(xform);
         Matrix m = new Matrix(at);
@@ -904,6 +918,9 @@ public class PageDrawer extends PDFGraphicsStreamEngine
 
                 case PathIterator.SEG_CLOSE:
                     break;
+
+                default:
+                    break;
             }
             iter.next();
         }
@@ -999,8 +1016,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
 
             // if the image is scaled down, we use smooth interpolation, eg PDFBOX-2364
             // only when scaled up do we use nearest neighbour, eg PDFBOX-2302 / mori-cvpr01.pdf
-            // stencils are excluded from this rule (see survey.pdf)
-            if (isScaledUp || pdImage.isStencil())
+            if (isScaledUp)
             {
                 graphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
                         RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
@@ -1025,10 +1041,22 @@ public class PageDrawer extends PDFGraphicsStreamEngine
                 Paint paint = getNonStrokingPaint();
                 Rectangle2D unitRect = new Rectangle2D.Float(0, 0, 1, 1);
                 Rectangle2D bounds = at.createTransformedShape(unitRect).getBounds2D();
-                BufferedImage renderedPaint = 
-                        new BufferedImage((int) Math.ceil(bounds.getWidth()), 
-                                          (int) Math.ceil(bounds.getHeight()), 
-                                           BufferedImage.TYPE_INT_ARGB);
+                GraphicsConfiguration deviceConfiguration = graphics.getDeviceConfiguration();
+                int w;
+                int h;
+                if (deviceConfiguration != null && deviceConfiguration.getBounds() != null)
+                {
+                    // PDFBOX-4690: bounds doesn't need to be larger than device bounds (OOM risk)
+                    Rectangle deviceBounds = deviceConfiguration.getBounds();
+                    w = (int) Math.ceil(Math.min(bounds.getWidth(), deviceBounds.getWidth()));
+                    h = (int) Math.ceil(Math.min(bounds.getHeight(), deviceBounds.getHeight()));
+                }
+                else
+                {
+                    w = (int) Math.ceil(bounds.getWidth());
+                    h = (int) Math.ceil(bounds.getHeight());
+                }
+                BufferedImage renderedPaint = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
                 Graphics2D g = (Graphics2D) renderedPaint.getGraphics();
                 g.translate(-bounds.getMinX(), -bounds.getMinY());
                 g.setPaint(paint);
@@ -1037,9 +1065,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
 
                 // draw the mask
                 BufferedImage mask = pdImage.getImage();
-                BufferedImage renderedMask = new BufferedImage((int) Math.ceil(bounds.getWidth()), 
-                                                               (int) Math.ceil(bounds.getHeight()), 
-                                                               BufferedImage.TYPE_INT_RGB);
+                BufferedImage renderedMask = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
                 g = (Graphics2D) renderedMask.getGraphics();
                 g.translate(-bounds.getMinX(), -bounds.getMinY());
                 AffineTransform imageTransform = new AffineTransform(at);
@@ -1053,8 +1079,6 @@ public class PageDrawer extends PDFGraphicsStreamEngine
                 int[] alphaPixel = null;
                 WritableRaster raster = renderedPaint.getRaster();
                 WritableRaster alpha = renderedMask.getRaster();
-                int h = renderedMask.getRaster().getHeight();
-                int w = renderedMask.getRaster().getWidth();
                 for (int y = 0; y < h; y++)
                 {
                     for (int x = 0; x < w; x++)
@@ -1195,8 +1219,12 @@ public class PageDrawer extends PDFGraphicsStreamEngine
 
         // prepare transfer functions (either one per color or one for all) 
         // and maps (actually arrays[256] to be faster) to avoid calculating values several times
-        Integer rMap[], gMap[], bMap[];
-        PDFunction rf, gf, bf;
+        Integer rMap[];
+        Integer gMap[];
+        Integer bMap[];
+        PDFunction rf;
+        PDFunction gf;
+        PDFunction bf;
         if (transfer instanceof COSArray)
         {
             COSArray ar = (COSArray) transfer;
@@ -1227,7 +1255,9 @@ public class PageDrawer extends PDFGraphicsStreamEngine
                 int ri = (rgb >> 16) & 0xFF;
                 int gi = (rgb >> 8) & 0xFF;
                 int bi = rgb & 0xFF;
-                int ro, go, bo;
+                int ro;
+                int go;
+                int bo;
                 if (rMap[ri] != null)
                 {
                     ro = rMap[ri];
@@ -1339,8 +1369,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         PDAppearanceDictionary appearance = annotation.getAppearance();
         if (appearance == null || appearance.getNormalAppearance() == null)
         {
-            // TODO: Improve memory consumption by passing a ScratchFile
-            annotation.constructAppearances();
+            annotation.constructAppearances(renderer.document);
         }
 
         if (annotation.isNoRotate() && getCurrentPage().getRotation() != 0)
@@ -1394,7 +1423,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         // both the DPI xform and the CTM were already applied to the group, so all we do
         // here is draw it directly onto the Graphics2D device at the appropriate position
         PDRectangle bbox = group.getBBox();
-        AffineTransform prev = graphics.getTransform();
+        AffineTransform savedTransform = graphics.getTransform();
 
         Matrix m = new Matrix(xform);
         float xScale = Math.abs(m.getScalingFactorX());
@@ -1435,11 +1464,19 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         {
             if (isContentRendered())
             {
-                graphics.drawImage(image, null, null);
+                try
+                {
+                    graphics.drawImage(image, null, null);
+                }
+                catch (InternalError ie)
+                {
+                    LOG.error("Exception drawing image, see JDK-6689349, " +
+                              "try rendering into a BufferedImage instead", ie);
+                }
             }
         }
 
-        graphics.setTransform(prev);
+        graphics.setTransform(savedTransform);
     }
 
     /**
@@ -1474,8 +1511,9 @@ public class PageDrawer extends PDFGraphicsStreamEngine
         private TransparencyGroup(PDTransparencyGroup form, boolean isSoftMask, Matrix ctm, 
                 PDColor backdropColor) throws IOException
         {
-            Graphics2D g2dOriginal = graphics;
-            Area lastClipOriginal = lastClip;
+            Graphics2D savedGraphics = graphics;
+            Area savedLastClip = lastClip;
+            Shape savedInitialClip = initialClip;
 
             // get the CTM x Form Matrix transform
             Matrix transform = Matrix.concatenate(ctm, form.getMatrix());
@@ -1573,7 +1611,7 @@ public class PageDrawer extends PDFGraphicsStreamEngine
             g.translate(0, image.getHeight());
             g.scale(1, -1);
 
-            boolean oldFlipTG = flipTG;
+            boolean savedFlipTG = flipTG;
             flipTG = false;
 
             // apply device transform (DPI)
@@ -1615,10 +1653,11 @@ public class PageDrawer extends PDFGraphicsStreamEngine
             }
             finally 
             {
-                flipTG = oldFlipTG;
-                lastClip = lastClipOriginal;
+                flipTG = savedFlipTG;
+                lastClip = savedLastClip;
                 graphics.dispose();
-                graphics = g2dOriginal;
+                graphics = savedGraphics;
+                initialClip = savedInitialClip;
                 clipWindingRule = clipWindingRuleOriginal;
                 linePath = linePathOriginal;
                 pageSize = pageSizeOriginal;
