@@ -924,7 +924,7 @@ public class PDFTextStripper extends LegacyPDFStreamEngine
 
             // In the wild, some PDF encoded documents put diacritics (accents on
             // top of characters) into a separate Tj element. When displaying them
-            // graphically, the two chunks get overlayed. With text output though,
+            // graphically, the two chunks get overlaid. With text output though,
             // we need to do the overlay. This code recombines the diacritic with
             // its associated character if the two are consecutive.
             if (textList.isEmpty())
@@ -1839,8 +1839,8 @@ public class PDFTextStripper extends LegacyPDFStreamEngine
 
     static
     {
-        String path = "org/apache/pdfbox/resources/text/BidiMirroring.txt";
-        InputStream input = PDFTextStripper.class.getClassLoader().getResourceAsStream(path);
+        String path = "/org/apache/pdfbox/resources/text/BidiMirroring.txt";
+        InputStream input = PDFTextStripper.class.getResourceAsStream(path);
         try
         {
             if (input != null)
