@@ -273,7 +273,7 @@ public final class Matrix implements Cloneable
     }
 
     /**
-     * Translates this matrix by the given ammount.
+     * Translates this matrix by the given amount.
      *
      * @param tx x-translation
      * @param ty y-translation
@@ -702,7 +702,10 @@ public final class Matrix implements Cloneable
     }
 
     /**
-     * Returns a COS array which represents this matrix.
+     * Returns a COS array which represent the geometric relevant
+     * components of the matrix. The last column of the matrix is ignored,
+     * only the first two columns are returned. This is analog to the
+     * Matrix(COSArray) constructor.
      */
     public COSArray toCOSArray()
     {
@@ -719,14 +722,13 @@ public final class Matrix implements Cloneable
     @Override
     public String toString()
     {
-        String sb = "" + "[" +
-                single[0] + "," +
-                single[1] + "," +
-                single[3] + "," +
-                single[4] + "," +
-                single[6] + "," +
-                single[7] + "]";
-        return sb;
+        return "[" +
+            single[0] + "," +
+            single[1] + "," +
+            single[3] + "," +
+            single[4] + "," +
+            single[6] + "," +
+            single[7] + "]";
     }
 
     @Override
