@@ -41,8 +41,8 @@ public class TestTTFParser
     @Test
     public void testPostTable() throws IOException
     {
-        InputStream input = PDFont.class.getClassLoader().getResourceAsStream(
-                "org/apache/pdfbox/resources/ttf/LiberationSans-Regular.ttf");
+        InputStream input = PDFont.class.getResourceAsStream(
+                "/org/apache/pdfbox/resources/ttf/LiberationSans-Regular.ttf");
         Assert.assertNotNull(input);
 
         TTFParser parser = new TTFParser();
