@@ -820,16 +820,13 @@ public class PreflightParser extends PDFParser
                     endObjectKey = readString();
 
                     // we have case with a second 'endstream' before endobj
-                    if (!endObjectKey.startsWith("endobj"))
+                    if (!endObjectKey.startsWith("endobj") && endObjectKey.startsWith("endstream"))
                     {
-                        if (endObjectKey.startsWith("endstream"))
+                        endObjectKey = endObjectKey.substring(9).trim();
+                        if (endObjectKey.length() == 0)
                         {
-                            endObjectKey = endObjectKey.substring(9).trim();
-                            if (endObjectKey.length() == 0)
-                            {
-                                // no other characters in extra endstream line
-                                endObjectKey = readString(); // read next line
-                            }
+                            // no other characters in extra endstream line
+                            endObjectKey = readString(); // read next line
                         }
                     }
                 }
@@ -913,7 +910,7 @@ public class PreflightParser extends PDFParser
                     {
                         position = source.getPosition();
                     }
-                    catch(IOException excpetion)
+                    catch (IOException ex)
                     {
                         position = Long.MIN_VALUE;
                     }
