@@ -251,7 +251,7 @@ public final class TypeMapping
         schemaMap.put(ns, new XMPSchemaFactory(ns, classSchem, initializePropMapping(classSchem)));
     }
 
-    public void addNewNameSpace(String ns, String prefered)
+    public void addNewNameSpace(String ns, String preferred)
     {
         PropertiesDescription mapping = new PropertiesDescription();
         schemaMap.put(ns, new XMPSchemaFactory(ns, XMPSchema.class, mapping));
@@ -369,7 +369,7 @@ public final class TypeMapping
                 catch (Exception e)
                 {
                     throw new IllegalArgumentException(
-                            "couldn't read one type declaration, please check accessibility and declaration of fields annoted in "
+                            "couldn't read one type declaration, please check accessibility and declaration of fields annotated in "
                                     + classSchem.getName(), e);
                 }
                 PropertyType propType = field.getAnnotation(PropertyType.class);
